Package: slam
Version: 0.1-55
Title: Sparse Lightweight Arrays and Matrices
Authors@R: c(person("Kurt", "Hornik", role = c("aut", "cre"),
                     email = "Kurt.Hornik@R-project.org",
		     comment = c(ORCID = "0000-0003-4198-9911")),
             person("David", "Meyer", role = "aut",
	             email = "David.Meyer@R-project.org",
		     comment = c(ORCID = "0000-0002-5196-3048")),
             person("Christian", "Buchta", role = "aut"))
Description: Data structures and algorithms for sparse arrays and matrices,
  based on index arrays and simple triplet representations, respectively.
Depends: R (>= 3.4.0)
Imports: stats
Enhances: Matrix, SparseM, spam
License: GPL-2
NeedsCompilation: yes
Packaged: 2024-11-12 16:45:23 UTC; hornik
Author: Kurt Hornik [aut, cre] (<https://orcid.org/0000-0003-4198-9911>),
  David Meyer [aut] (<https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2024-11-13 07:13:31 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:56:17 UTC; unix
Archs: slam.so.dSYM

Package: slam
Version: 0.1-55
Title: Sparse Lightweight Arrays and Matrices
Authors@R: c(person("Kurt", "Hornik", role = c("aut", "cre"),
                     email = "Kurt.Hornik@R-project.org",
		     comment = c(ORCID = "0000-0003-4198-9911")),
             person("David", "Meyer", role = "aut",
	             email = "David.Meyer@R-project.org",
		     comment = c(ORCID = "0000-0002-5196-3048")),
             person("Christian", "Buchta", role = "aut"))
Description: Data structures and algorithms for sparse arrays and matrices,
  based on index arrays and simple triplet representations, respectively.
Depends: R (>= 3.4.0)
Imports: stats
Enhances: Matrix, SparseM, spam
License: GPL-2
NeedsCompilation: yes
Packaged: 2024-11-12 16:45:23 UTC; hornik
Author: Kurt Hornik [aut, cre] (<https://orcid.org/0000-0003-4198-9911>),
  David Meyer [aut] (<https://orcid.org/0000-0002-5196-3048>),
  Christian Buchta [aut]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2024-11-13 07:13:31 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-18 06:33:51 UTC; unix

importFrom("stats", "na.omit")

## Simple triplet matrix stuff
export("as.simple_triplet_matrix",
       "is.simple_triplet_matrix",
       "simple_triplet_diag_matrix",
       "simple_triplet_matrix",
       "simple_triplet_zero_matrix"
       )

S3method("[", "simple_triplet_matrix")
S3method("[<-", "simple_triplet_matrix")
S3method("Math", "simple_triplet_matrix")
S3method("Ops", "simple_triplet_matrix")
S3method("Summary", "simple_triplet_matrix")
S3method("aperm", "simple_triplet_matrix")
S3method("as.matrix", "simple_triplet_matrix")
S3method("as.simple_triplet_matrix", "simple_sparse_array")
S3method("as.simple_triplet_matrix", "simple_triplet_matrix")
S3method("as.simple_triplet_matrix", "matrix")
S3method("as.simple_triplet_matrix", "default")
S3method("as.simple_triplet_matrix", "dgTMatrix")
S3method("as.simple_triplet_matrix", "dgCMatrix")
S3method("as.simple_triplet_matrix", "dgRMatrix")
S3method("as.simple_triplet_matrix", "matrix.coo")
S3method("as.simple_triplet_matrix", "matrix.csr")
S3method("as.simple_triplet_matrix", "matrix.csc")
S3method("as.simple_triplet_matrix", "spam")
S3method("as.vector", "simple_triplet_matrix")
S3method("c", "simple_triplet_matrix")
S3method("cbind", "simple_triplet_matrix")
S3method("dim", "simple_triplet_matrix")
S3method("dim<-", "simple_triplet_matrix")
S3method("dimnames", "simple_triplet_matrix")
S3method("dimnames<-", "simple_triplet_matrix")
S3method("duplicated", "simple_triplet_matrix")
S3method("is.numeric", "simple_triplet_matrix")
S3method("mean", "simple_triplet_matrix")
S3method("print", "simple_triplet_matrix")
S3method("rbind", "simple_triplet_matrix")
S3method("split", "simple_triplet_matrix")
S3method("t", "simple_triplet_matrix")
S3method("unique", "simple_triplet_matrix")

S3method(".is_sparse_mat_coercible_to_stm", "simple_triplet_matrix")
S3method(".is_sparse_mat_coercible_to_stm", "default")
S3method(".is_sparse_mat_coercible_to_stm", "dgTMatrix")
S3method(".is_sparse_mat_coercible_to_stm", "dgCMatrix")
S3method(".is_sparse_mat_coercible_to_stm", "dgRMatrix")
S3method(".is_sparse_mat_coercible_to_stm", "matrix.coo")
S3method(".is_sparse_mat_coercible_to_stm", "matrix.csr")
S3method(".is_sparse_mat_coercible_to_stm", "matrix.csc")
S3method(".is_sparse_mat_coercible_to_stm", "spam")

## enhanced stuff
useDynLib("slam", .registration = TRUE)

export("row_sums",
       "col_sums",
       "row_means",
       "col_means"
       )

S3method("row_sums", "default")
S3method("row_sums", "simple_triplet_matrix")
S3method("row_sums", "dgTMatrix")
S3method("row_sums", "dgCMatrix")

S3method("col_sums", "default")
S3method("col_sums", "simple_triplet_matrix")
S3method("col_sums", "dgTMatrix")
S3method("col_sums", "dgCMatrix")

S3method("row_means", "default")
S3method("row_means", "simple_triplet_matrix")
S3method("row_means", "dgTMatrix")
S3method("row_means", "dgCMatrix")

S3method("col_means", "default")
S3method("col_means", "simple_triplet_matrix")
S3method("col_means", "dgTMatrix")
S3method("col_means", "dgCMatrix")

export("row_norms",
       "col_norms")

##
export("tcrossprod_simple_triplet_matrix",
       "crossprod_simple_triplet_matrix",
       "matprod_simple_triplet_matrix")

if(getRversion() >= "4.3.0") {
    S3method("matrixOps", "simple_triplet_matrix")
    S3method("chooseOpsMethod", "simple_triplet_matrix")
}

export("rowapply_simple_triplet_matrix",
       "colapply_simple_triplet_matrix",
       "crossapply_simple_triplet_matrix",
       "tcrossapply_simple_triplet_matrix")

##
export("rollup")

S3method("rollup", "default")
S3method("rollup", "matrix")
S3method("rollup", "array")
S3method("rollup", "simple_sparse_array")
S3method("rollup", "simple_triplet_matrix")

## Simple sparse array stuff
export("as.simple_sparse_array",
       "is.simple_sparse_array",
       "simple_sparse_array",
       "simple_sparse_zero_array",
       ##
       "simplify_simple_sparse_array",
       "reduce_simple_sparse_array",
       "drop_simple_sparse_array",
       ##
       "extend_simple_sparse_array",
       "abind_simple_sparse_array"
       )

S3method("[", "simple_sparse_array")
S3method("[<-", "simple_sparse_array")
S3method("Math", "simple_sparse_array")
S3method("Summary", "simple_sparse_array")
S3method("aperm", "simple_sparse_array")
S3method("as.array", "simple_sparse_array")
S3method("as.array", "simple_triplet_matrix")
S3method("as.simple_sparse_array", "simple_sparse_array")
S3method("as.simple_sparse_array", "simple_triplet_matrix")
S3method("as.simple_sparse_array", "array")
S3method("as.simple_sparse_array", "matrix")
S3method("as.simple_sparse_array", "default")
S3method("as.vector", "simple_sparse_array")
S3method("dim", "simple_sparse_array")
S3method("dim<-", "simple_sparse_array")
S3method("dimnames", "simple_sparse_array")
S3method("dimnames<-", "simple_sparse_array")
S3method("is.numeric", "simple_sparse_array")
S3method("mean", "simple_sparse_array")
S3method("print", "simple_sparse_array")

## Sparse matrix format readers and writers
export("read_stm_CLUTO",
       "write_stm_CLUTO",
       "read_stm_MC",
       "write_stm_MC"
       )

##
## export("unfold",
##        "fold")
## S3method("unfold", "default")
## S3method("unfold", "matrix")
## S3method("unfold", "array")
## S3method("unfold", "simple_triplet_matrix")
## S3method("unfold", "simple_sparse_array")
## S3method("fold",   "default")
## S3method("fold",   "matrix")
## S3method("fold",   "array")
## S3method("fold",   "simple_triplet_matrix")
## S3method("fold",   "simple_sparse_array")

##
export("slam_options")

# slam

[![Build Status](https://travis-ci.org/miguelgrinberg/slam.png?branch=master)](https://travis-ci.org/miguelgrinberg/slam)

Slam is a simple command-line utility that makes it very easy to do serverless
deployments of Python web applications to  [AWS](https://aws.amazon.com), using
the [Lambda](https://aws.amazon.com/lambda) and
[API Gateway](https://aws.amazon.com/api-gateway) services. In particular, Slam
supports the transparent deployment of [WSGI](https://www.python.org/dev/peps/pep-3333/)
compliant applications.

If you have 10 minutes to spare, you can watch a (now slightly outdated)
screencast in which the main features of slam are demonstrated:

[![Slam Demonstration](https://img.youtube.com/vi/9eoL6oGiodw/0.jpg)](https://www.youtube.com/watch?v=9eoL6oGiodw)

Or if you prefer something more formal, have a look at the 
[documentation](http://slam-python.readthedocs.io/en/latest/), which also
includes a written tutorial.

##
library("slam")

s <- as.simple_triplet_matrix(diag(4))
s[1:8] <- 1:8
as.matrix(s)

s[2:3,] <- 1:8
as.matrix(s)

s[,2:3] <- 1:8
as.matrix(s)

s[] <- 1:8
as.matrix(s)

##
local({
    k <- 2:3
    ## Implementing class.
    a <- as.simple_sparse_array(s)
    a[,k]
    a[,k] <- 1:8
    s[,k] <- 1:8
    stopifnot(identical(as.array(a), as.array(s)))
})

###

require("slam")

x <- matrix(1 : 8, 2, 4)
dimnames(x) <- list(ROW = LETTERS[seq_len(nrow(x))],
                    COL = letters[seq_len(ncol(x))])
s <- as.simple_triplet_matrix(x)
dim(s) <- dim(x) <- c(4, 2)
stopifnot(identical(as.matrix(s), x))

d <- c(2, 3, 4)
x <- array(seq_len(prod(d)), d)
s <- as.simple_sparse_array(x)
dim(s) <- dim(x) <- c(d[length(d)], d[-length(d)])
stopifnot(identical(as.array(s), x))

dimnames(s) <- dimnames(x) <- NULL
stopifnot(identical(as.array(s), x))

library("slam")
set.seed(201311)

###
x <- matrix(rnorm(100), nrow = 20,
    dimnames =  list(1:20, LETTERS[1:5])
)
x[sample(100, 80)] <- 0

s <- as.simple_triplet_matrix(x)
s

##
identical(apply(x, 2L, var), colapply_simple_triplet_matrix(s, var))
identical(apply(x, 1L, var), rowapply_simple_triplet_matrix(s, var))

local({
    x[] <- as.complex(x)
    s   <- as.simple_triplet_matrix(x)
    identical(apply(x, 2L, var), colapply_simple_triplet_matrix(s, var))
})

##
k <- 1:2
z <- var(x[, k], x[, -k])
identical(z, crossapply_simple_triplet_matrix(s[, k], s[, -k], FUN = var))
identical(z, crossapply_simple_triplet_matrix(x[, k], s[, -k], FUN = var))

identical(z, 
    tcrossapply_simple_triplet_matrix(t(s[, k]), t(s[, -k]), FUN = var))
identical(z, 
    tcrossapply_simple_triplet_matrix(t(x[, k]), t(s[, -k]), FUN = var))

z <- var(x)
identical(z, crossapply_simple_triplet_matrix(s, FUN = var))

## null-dimensions
z <- var(x[, 0], x)
z
all.equal(z, crossapply_simple_triplet_matrix(s[, 0], s, FUN = var))
all.equal(z, crossapply_simple_triplet_matrix(x[, 0], s, FUN = var))

try(crossapply_simple_triplet_matrix(x[, 0], s, FUN = var, use = "all.obs"))

z <- var(x, x[, 0])
z
all.equal(z, crossapply_simple_triplet_matrix(s, s[, 0], FUN = var))
all.equal(z, crossapply_simple_triplet_matrix(x, s[, 0], FUN = var))


z <- var(x[, 0])
z
all.equal(z, crossapply_simple_triplet_matrix(s[, 0], s[, 0], FUN = var))
all.equal(z, crossapply_simple_triplet_matrix(x[, 0], s[, 0], FUN = var))

all.equal(z, crossapply_simple_triplet_matrix(s[, 0], FUN = var))

z <- var(x[0, ])
z
all.equal(z, crossapply_simple_triplet_matrix(s[0, ], s[0, ], FUN = var))
all.equal(z, crossapply_simple_triplet_matrix(x[0, ], s[0, ], FUN = var))

all.equal(z, crossapply_simple_triplet_matrix(s[0, ], FUN = var))

## non-scalar
z <- crossapply_simple_triplet_matrix(s, s, FUN = ">")
all.equal(z, crossapply_simple_triplet_matrix(x, s, FUN = ">"))

all.equal(z[lower.tri(z)],
    crossapply_simple_triplet_matrix(s, FUN = ">")[lower.tri(z)])

###

##
library("slam")
##
x <- simple_sparse_zero_array(dim = c(3, 2))

x[1]
x[matrix(c(1, 1), nrow = 1)]

##
x <- as.simple_sparse_array(matrix(1:6, ncol = 2))
x[1]
x[matrix(c(1, 1), nrow = 1)]

x[1.1]				## truncation

x[integer()]
x[matrix(integer(), ncol = 2)]


				## missing values
x[c(1, 0, NA, 2)]

k <- matrix(c(1, 1, 1, 0, 1, NA), ncol = 2, byrow = TRUE)
k
x[k]

try(x[as.logical(k)])		## wrong type
				## wrong dimensions 
dim(k) <- c(2,3)
as.vector(k)
x[k]


z <- x[c(1,3),]
data.frame(v = z$v, i = z$i,
    k = .Call(slam:::R_vector_index, z$dim, z$i))


## drop not implemented
x[ 1,]
x[-1,]
try(x[1, NA_integer_]) 		## not implemented
str(x[0,])
str(x[0, 0])

x[c(1, 8)]			## out of bounds allowed
try(x[1, 8])			## not allowed

dim(k) <- c(3,2)
k[6] <- 3
k
try(x[k])			## not allowed

x[cbind(c(0, 1), c(-1, 0))]	## allowed

##
x <- simple_triplet_zero_matrix(nrow = 3, ncol = 2)

x[1]
x[matrix(c(1, 1), nrow = 1)]

##
x <- as.simple_triplet_matrix(matrix(1:6, ncol = 2))
x[1]
x[matrix(c(1, 1), nrow = 1)]

x[1.1]				## truncation

x[integer()]
x[matrix(integer(), ncol = 2)]


				## missing values
x[c(1, 0, NA, 2)]

k <- matrix(c(1, 1, 1, 0, 1, NA), ncol = 2, byrow = TRUE)
k
x[k]

x[as.logical(k)]		## wrong type
				## wrong dimensions 
dim(k) <- c(2,3)
as.vector(k)
x[k]


z <- x[c(1,3),]
data.frame(v = z$v, i = z$i, j = z$j,
    k = .Call(slam:::R_vector_index, c(z$nrow, z$ncol), cbind(z$i, z$j)))


## drop not implemented
x[ 1,]
x[-1,]
try(x[1, NA_integer_]) 		## not implemented
str(x[0,])
str(x[0, 0])

x[c(1, 8)]			## out of bounds allowed
try(x[1, 8])			## not allowed

dim(k) <- c(3,2)
k[6] <- 3
k
try(x[k])			## not allowed

x[cbind(c(0, 1), c(-1, 0))]	## allowed

x[c(TRUE, FALSE)]
x[c(TRUE, FALSE),]

## reference
x <- matrix(1:6, ncol = 2)
x[c(1, 0, NA, 2)]

try(x[-c(1, NA)])		## not allowed

				## missing allowed
k <- matrix(c(1, 1, 1, 0, 1, NA), ncol = 2, byrow = TRUE)
k
x[k]

dim(k) <- c(2, 3)
as.vector(k)
x[k]


x[ 1,]
x[-1,]
x[ 1, NA]			## wildcard

x[0,]				## does not drop
x[0,0]

x[c(1, 8)]			## out of bounds allowed
try(x[1, 8])			## not allowed

dim(k) <- c(3,2)
k[6] <- 3
k
try(x[k])			## not allowed

x[c(TRUE, FALSE)]
x[c(TRUE, FALSE),]


##
m <- matrix(c(1, 1, 0, 2), nrow = 2)
m
s <- as.simple_triplet_matrix(m)
identical(s[s > 0], m[m > 0])
identical(s[s > 0], s$v)	## not guaranteed
local({
    s[s > 0] <- 3
    m[m > 0] <- 3
    identical(as.matrix(s), m)
})

try(s[s])
a <- as.simple_sparse_array(s)
try(s[a])

is.na(m) <- 2
m
s <- as.simple_triplet_matrix(m)
identical(s[s > 0], m[m > 0])

local({
    s[s > 0] <- 3
    m[m > 0] <- 3
    identical(as.matrix(s), m)
})

###

library("slam")
## <FIXME>
## Remove eventually.
suppressWarnings(RNGversion("3.5.0"))
## </FIXME>
set.seed(20090626)

###

x <- sample(0:5, 100, T, prob=c(.8,rep(.04,5)))
x <- matrix(as.logical(x), nrow = 20,
     dimnames = list(rows = 1:20, cols = LETTERS[1:5]))
x

xst <- as.simple_triplet_matrix(x)
xst

identical(rowSums(x), row_sums(xst))
identical(colSums(x), col_sums(xst))
identical(rowMeans(x), row_means(xst))
identical(colMeans(x), col_means(xst))

local({
    x[] <- as.double(x)
    xst <- as.simple_triplet_matrix(x)
    identical(rowSums(x), row_sums(xst))
})

local({
    x[] <- as.complex(x)
    xst <- as.simple_triplet_matrix(x)
    identical(rowSums(x), row_sums(xst))
})

## NAs

xna <- x
n <- prod(dim(x))
is.na(xna) <- sample(seq_len(n), ceiling(n * .1))
xna

xnast <- as.simple_triplet_matrix(xna)
xnast

## default method
identical(rowSums(xna), row_sums(xna))
identical(colSums(xna), col_sums(xna))
identical(rowMeans(xna), row_means(xna))
identical(colMeans(xna), col_means(xna))

identical(rowSums(xna), row_sums(xnast))
identical(colSums(xna), col_sums(xnast))
identical(rowMeans(xna), row_means(xnast))
identical(colMeans(xna), col_means(xnast))

local({
    xna[] <- as.double(xna)
    xnast <- as.simple_triplet_matrix(xna)
    identical(rowSums(xna), row_sums(xnast))
})

local({
    xna[] <- as.complex(xna)
    xnast <- as.simple_triplet_matrix(xna)
    identical(rowSums(xna), row_sums(xnast))
})

identical(rowSums(xna, na.rm = TRUE), row_sums(xnast, na.rm = TRUE))
identical(colSums(xna, na.rm = TRUE), col_sums(xnast, na.rm = TRUE))
identical(rowMeans(xna, na.rm = TRUE), row_means(xnast, na.rm = TRUE))
identical(colMeans(xna, na.rm = TRUE), col_means(xnast, na.rm = TRUE))

local({
    xna[] <- as.double(xna)
    xnast <- as.simple_triplet_matrix(xna)
    identical(rowSums(xna, na.rm = TRUE), row_sums(xnast, na.rm = TRUE))
})

local({
    xna[] <- as.complex(xna)
    xnast <- as.simple_triplet_matrix(xna)
    identical(rowSums(xna, na.rm = TRUE), row_sums(xnast, na.rm = TRUE))
})

## cross-product

identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(xst))
identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(xst, x))
identical(tcrossprod(x[1:10,], x[11:20,]),
	  tcrossprod_simple_triplet_matrix(xst[1:10,], xst[11:20,]))

x <- matrix(c(1, 0, 0, 2, 1, NA), nrow = 3)
x
s <- as.simple_triplet_matrix(x)

identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(s))
identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(s, x))
identical(tcrossprod(x[2:3,], x[1,, drop = FALSE]), 
	  tcrossprod_simple_triplet_matrix(s[2:3,], s[1,]))
identical(tcrossprod(x[1,, drop = FALSE], x[2:3,]), 
	  tcrossprod_simple_triplet_matrix(s[1,], s[2:3,]))

###

library("slam")

s <- simple_triplet_diag_matrix(1, nrow = 3)

identical(as.matrix(s) * Inf, as.matrix(s * Inf))
identical(as.matrix(s) *  NA, as.matrix(s *  NA_real_))

identical(as.matrix(s) * c(Inf, NA, 0), as.matrix(s * c(Inf, NA, 0)))

x1 <- matrix(c(1, Inf, 0, 1), nrow = 2)
x2 <- matrix(c(1, 0,  NA, 1), nrow = 2)

identical(x1 * x2, as.matrix(as.simple_triplet_matrix(x1) * x2))
identical(x1 * x2, as.matrix(as.simple_triplet_matrix(x1) *
                             as.simple_triplet_matrix(x2)))

x <- matrix(1, nrow = 3, ncol = 3)
identical(x * as.matrix(s), as.matrix(s * as.simple_triplet_matrix(x)))
identical(x / as.matrix(s), as.matrix(as.simple_triplet_matrix(x) / s))

identical(x * as.matrix(s), as.matrix(s * x))
identical(x / as.matrix(s), as.matrix(x / s))

###

library("slam")

## test
x <- matrix(c(1, 0, 0, 2, 1, 0), nrow = 3, 
    dimnames = list(A = 1:3, B = 1:2))
s <- as.simple_triplet_matrix(x)
dimnames(x)[[1L]] <- letters[1:3]
names(dimnames(x))[1L] <- 1
x

##
z <- tcrossprod_simple_triplet_matrix(s, x[1:2,])
z

zz <- slam:::.ttcrossprod_simple_triplet_matrix(s, x[1:2,])
identical(z, t(zz))

## bailout
is.na(x) <- 4L

z <- tcrossprod_simple_triplet_matrix(s, x[1:2,])
z

zz <- slam:::.ttcrossprod_simple_triplet_matrix(s, x[1:2,])
identical(z, t(zz))

###

library("slam")
set.seed(201008)

## test
x <- matrix(sample(c(0,1), 100L, TRUE, prob = c(.9,.1)), 5L)
dim(x)
INDEX <- sample(1:4, 20L, TRUE)

s <- as.simple_triplet_matrix(x)
z <- as.matrix(s)

rollup(z, 2L, rep(1L, 20L), fivenum)

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
identical(rollup(t(z), 1L, INDEX), 
	  as.matrix(rollup(t(s), 1L, INDEX)))

## NA indexes
k <- INDEX
is.na(k) <- k == 1L
any(is.na(k))
identical(as.matrix(rollup(s, 2L, k)), 
	  rollup(z, 2L, k))

## other data types
s$v <- as.integer(s$v)

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
##
local({
    s$v <- as.complex(s$v)
    z   <- as.matrix(s)
    identical(rollup(z, 2L, INDEX), 
	      as.matrix(rollup(s, 2L, INDEX)))
})

## NA values
is.na(s$v) <- 1:2
z   <- as.matrix(s)
z[] <- as.double(z) # coerce

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	  as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
s$v <- as.double(s$v)

identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	  as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
local({
    s$v <- as.complex(s$v)
    z   <- as.matrix(s)
    identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	      as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))
})

##
s <- as.simple_sparse_array(s)
z <- as.array(z)

identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
INDEX <- rep(1, dim(x)[2L])

identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

s <- as.simple_triplet_matrix(s)
identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

## reduce
is.na(s$v) <- s$i == 1L

z <- rollup(as.simple_sparse_array(s), 2L, na.rm = TRUE)
z <- reduce_simple_sparse_array(z, order = TRUE)
z <- as.simple_triplet_matrix(z)
identical(z,
	  .Call(slam:::R_row_tsums, s, rep(factor(1L), ncol(s)), 
		TRUE, TRUE, TRUE))

s$v <- as.complex(s$v)

z <- rollup(as.simple_sparse_array(s), 2L, na.rm = TRUE)
z <- reduce_simple_sparse_array(z, order = TRUE)
z <- as.simple_triplet_matrix(z)
identical(z, 
	  rollup(s, 2L, na.rm = TRUE, REDUCE = TRUE))
###

##
library("slam")
## sparse
x <- simple_sparse_zero_array(dim = c(3, 4, 2))
				    ## removal of subscripts
k <- matrix(c(2, 1, 1, 0, 1, 1), c(2, 3), byrow = TRUE)
k
x[k] <- 1
x[3, 1, 1] <- 2 
x[c(17, 17)] <- c(2, 3)		    ## duplicate subscripts
x[c(5, NA, 6)] <- 4		    ## recycling
data.frame(v = x$v, i = x$i,
    k = .Call(slam:::R_vector_index, x$dim, x$i))

##
x[, -1, 1] <- 0			    ## zero elements
data.frame(v = x$v, i = x$i,
    k = .Call(slam:::R_vector_index, x$dim, x$i))
x[-c(2, 3)] <- 0
data.frame(v = x$v, i = x$i,
    k = .Call(slam:::R_vector_index, x$dim, x$i))

x[] <- 0
str(x)

## misc
x[integer()] <- 1
x[matrix(integer(), nrow = 0, ncol = 3)] <- 1
str(x)

try(x[c(NA, 2, 3)] <- 1:2)	    ## not allowed
## works with R >= 3.x
try(x[-c(.Machine$integer.max + 1, 1)] <- c(1, 2))

as.vector(x[1,1,1])
x[1L] <- NA
as.vector(x[1,1,1])

x[1L] <- 2
as.vector(x[1,1,1])

##
z <- drop_simple_sparse_array(x[1,,])
as.vector(z[1,])
z[1,] <- -as.simple_triplet_matrix(z[1,])
as.vector(z[1,])

## reference
x <- matrix(1:6, nrow = 3)

## matrix indexing
k <- matrix(c(1, 1, 2, 2, 1, 1), ncol = 2, byrow = TRUE)
k

z <- x
z[k] <- -1
z

z <- x
z[k] <- -(1:3)			    ## last in sequence 
z

## implicit vector indexing
k <- matrix(k, nrow = 2)
as.vector(k)

z <- x
z[k] <- -1
z

z <- x
z[k] <- -(1:6)			    ## last in sequence
z

## missing values
z <- x
z[c(NA, 1, 2)] <- -1
z

z <- x
try(z[c(NA, 1, 2)] <- -(1:2))	    ## not allowed

k[1L] <- NA			    ## implicit vector indexing
as.vector(k)
z <- x
z[k] <- -1
z

k <- matrix(c(NA, 1, 1, 1, 2, 2), ncol = 2, byrow = TRUE)
k

z <- x
z[k] <- -1
z

z <- x
try(z[k] <- -(1:2))		    ## not allowed

## zeros
z <- x
z[c(0, 1)] <- -1
z

z <- x
z[c(0, 1)] <- -(1:2)
z

k <- matrix(c(1, 1, 0, 2), ncol = 2, byrow = TRUE)
k

z <- x
z[k] <- -1
z

z <- x
z[k] <- -(1:2)
z

## extending
k <- matrix(c(1, 4), ncol = 2)

z <- x
try(z[k] <- 1)			    ## not allowed

z[c(1, 8)] <- 1			    ## not implemented
z

## misc
z <- x
try(z[-c(.Machine$integer.max + 1, 1)] <- c(1, 2))

###

library("slam")
## <FIXME>
## Remove eventually.
suppressWarnings(RNGversion("3.5.0"))
## </FIXME>
set.seed(20091012)

###
as.simple_triplet_matrix_zeros <-
function(x) {
    x <- list(
        i        = rep(seq_len(nrow(x)), ncol(x)),
        j        = rep(seq_len(ncol(x)), each = nrow(x)),
        v        = c(x),
        nrow     = nrow(x),
        ncol     = ncol(x),
        dimnames = dimnames(x)
    )
    class(x) <-  "simple_triplet_matrix"
    x
}

x <- sample(0:5, 100, T, prob=c(.8,rep(.04,5)))
x <- matrix(as.logical(x), nrow = 20,
     dimnames = list(rows = 1:20, cols = LETTERS[1:5]))
x

xst <- as.simple_triplet_matrix_zeros(x)
xst

identical(rowSums(x), row_sums(xst))
identical(colSums(x), col_sums(xst))
identical(rowMeans(x), row_means(xst))
identical(colMeans(x), col_means(xst))

## NAs

xna <- x
n <- prod(dim(x))
is.na(xna) <- sample(seq_len(n), ceiling(n * .1))
xna

xnast <- as.simple_triplet_matrix_zeros(xna)
xnast

identical(rowSums(xna), row_sums(xnast))
identical(colSums(xna), col_sums(xnast))
identical(rowMeans(xna), row_means(xnast))
identical(colMeans(xna), col_means(xnast))

identical(rowSums(xna, na.rm = TRUE), row_sums(xnast, na.rm = TRUE))
identical(colSums(xna, na.rm = TRUE), col_sums(xnast, na.rm = TRUE))
identical(rowMeans(xna, na.rm = TRUE), row_means(xnast, na.rm = TRUE))
identical(colMeans(xna, na.rm = TRUE), col_means(xnast, na.rm = TRUE))

## cross-product

identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(xst))
identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(xst, x))

x <- matrix(c(1, 0, 0, 2, 1, NA), nrow = 3)
x
s <- as.simple_triplet_matrix_zeros(x)

identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(s))
identical(tcrossprod(x), tcrossprod_simple_triplet_matrix(s, x))

##
identical(as.matrix(s * x), x * x)
identical(as.matrix(x * s), x * x)
identical(as.matrix(s * s), x * x)

identical(as.matrix(s + s), x + x)

###

library("slam")
set.seed(201008)

## test
x <- matrix(sample(c(0,1), 100L, TRUE, prob = c(.9,.1)), 5L)
dim(x)
INDEX <- sample(1:4, 20L, TRUE)

s <- as.simple_triplet_matrix(x)
z <- as.matrix(s)

rollup(z, 2L, rep(1L, 20L), fivenum)

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
identical(rollup(t(z), 1L, INDEX), 
	  as.matrix(rollup(t(s), 1L, INDEX)))

## NA indexes
k <- INDEX
is.na(k) <- k == 1L
any(is.na(k))
identical(as.matrix(rollup(s, 2L, k)), 
	  rollup(z, 2L, k))

## other data types
s$v <- as.integer(s$v)

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
##
local({
    s$v <- as.complex(s$v)
    z   <- as.matrix(s)
    identical(rollup(z, 2L, INDEX), 
	      as.matrix(rollup(s, 2L, INDEX)))
})

## NA values
is.na(s$v) <- 1:2
z   <- as.matrix(s)
z[] <- as.double(z) # coerce

identical(rollup(z, 2L, INDEX), 
	  as.matrix(rollup(s, 2L, INDEX)))
identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	  as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
s$v <- as.double(s$v)

identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	  as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
local({
    s$v <- as.complex(s$v)
    z   <- as.matrix(s)
    identical(rollup(z, 2L, INDEX, na.rm = TRUE), 
	      as.matrix(rollup(s, 2L, INDEX, na.rm = TRUE)))
})

##
s <- as.simple_sparse_array(s)
z <- as.array(z)

identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

##
INDEX <- rep(1, dim(x)[2L])

identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

s <- as.simple_triplet_matrix(s)
identical(rollup(z, 2L, INDEX, na.rm = TRUE),
	  as.array(rollup(s, 2L, INDEX, na.rm = TRUE)))

## reduce
is.na(s$v) <- s$i == 1L

z <- rollup(as.simple_sparse_array(s), 2L, na.rm = TRUE)
z <- reduce_simple_sparse_array(z, order = TRUE)
z <- as.simple_triplet_matrix(z)
identical(z,
	  .Call(slam:::R_row_tsums, s, rep(factor(1L), ncol(s)), 
		TRUE, TRUE, FALSE))

s$v <- as.complex(s$v)

z <- rollup(as.simple_sparse_array(s), 2L, na.rm = TRUE)
z <- reduce_simple_sparse_array(z, order = TRUE)
z <- as.simple_triplet_matrix(z)
identical(z, 
	  rollup(s, 2L, na.rm = TRUE, REDUCE = TRUE))
###

###

rollup <- 
function(x, MARGIN, INDEX, FUN, ...)
    UseMethod("rollup")

rollup.array <-
function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
	    MODE = "double") {
    if (is.character(MARGIN))
        MARGIN <- match(MARGIN, names(dimnames(x)))
    if (!all(match(MARGIN, seq_along(dim(x)), nomatch = 0L)))
        stop("'MARGIN' invalid")
    if (is.null(INDEX))
	INDEX <- vector("list", length(MARGIN)) 
    else {
	if (is.atomic(INDEX))
	    INDEX <- list(INDEX)
	if (length(INDEX) != length(MARGIN))
	    stop("'INDEX' invalid length")
    }
    names(INDEX) <- MARGIN
    FUN <- match.fun(FUN)
    d <- dim(x)
    n <- dimnames(x)
    if (is.null(n))
	n <- vector("list", length(d))
    i <- arrayInd(seq_along(x), .dim = d)
    for (k in MARGIN) {
	z <- INDEX[[as.character(k)]]
	z <-
	if (is.null(z))
	    rep.int(as.factor(1L), d[k])
	else {
	    if (length(z) != d[k])
		stop(gettextf("INDEX [%s] invalid length", k),
                     domain = NA)
	    as.factor(z)
	}
	i[, k] <- z[i[, k]]
	z <- levels(z)
	d[k]   <- length(z)
	n[[k]] <- z
	rm(z)
    }
    i <- .Call(R_vector_index, d, i)
    attributes(i) <-
	list(levels = seq_len(prod(d)),
	     class  = "factor")
    i <- split.default(x, i)
    names(i) <- NULL
    i <- lapply(i, FUN, ...)
    if (all(unlist(lapply(i, length)) == 1L)) {
	i <- unlist(i, recursive = FALSE, use.names = FALSE)
	if (is.null(i))
	    i <- vector(MODE, 0L)
    }
    ## NOTE see drop_simple_sparse_array
    if (DROP) {
	if (any(d == 0L))
	    return(i)
	k <- which(d == 1L)
	if (length(k) == length(d))
	    return(i)
	if (length(k)) {
	    k <- -k
	    d <- d[k]
	    n <- n[k]
	}
    }
    array(i, d, n)
}

rollup.matrix <- rollup.array

rollup.simple_sparse_array <-
function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE,
	    EXPAND = c("none", "sparse", "dense", "all"), MODE = "double") {
    if (is.character(MARGIN)) 
	MARGIN <- match(MARGIN, names(dimnames(x)))
    if (!all(match(MARGIN, seq_along(dim(x)), nomatch = 0L)))
	stop("'MARGIN' invalid")
    if (is.null(INDEX))
	INDEX <- vector("list", length(MARGIN)) 
    else {
	if (is.atomic(INDEX))
	    INDEX <- list(INDEX)
	if (length(INDEX) != length(MARGIN))
	    stop("'INDEX' invalid length")
    }
    names(INDEX) <- MARGIN
    FUN <- match.fun(FUN)
    EXPAND <- match(
	match.arg(EXPAND), 
	eval(formals(rollup.simple_sparse_array)$EXPAND)
    )
    D <- dim(x)
    I <- x$i
    if (EXPAND > 1L) {
	if (EXPAND > 2L)
	    P <- array(1L, dim(I))
	T <- vector("list", length(D))
	for (k in seq_along(D)[-MARGIN])
	    T[[k]] <- rep.int(1L, D[k])
    }
    N <- dimnames(x)
    if (is.null(N))
	N <- vector("list", length(D))
    V <- x$v
    if (EXPAND < 4L &&
	!.Call(R__valid_v, V))
	stop("component 'v' contains 'ZERO' value(s)")
    for (k in MARGIN) {
	z <- INDEX[[as.character(k)]]
	if (is.null(z)) {
	    ## NOTE defer processing.
	    if (EXPAND < 3L) {
		if (EXPAND > 1L)
		    T[[k]] <- D[k]
		D[k] <- -1L
		next
	    }
	    z <- rep.int(as.factor(1L), D[k])
	} else {
	    if (length(z) != D[k])
		stop(gettextf("INDEX [%s] invalid length", k),
                     domain = NA)
	    z <- as.factor(z)
	}
	l <- levels(z)
	D[k]   <- length(l)
	N[[k]] <- l
	i <- I[, k]
	if (EXPAND > 1L) {
	    if (EXPAND > 2L) {
		p <- .Call(R_part_index, z)
		T[[k]] <- attr(p, "table")
		P[, k] <- p[i]
		rm(p)
	    } else
		T[[k]] <- tabulate(z, length(l))
	}
	i <- z[i]
	rm(l, z)
	I[, k] <- i
	i <- is.na(i)
	i <- which(i)
	if (length(i)) {
	    i <- - i
	    I <- I[i,, drop = FALSE]
	    V <- V[i]
	    if (EXPAND > 2L)
		P <- P[i,, drop = FALSE]
	}
	rm(i)
    }
    if (EXPAND == 4L) {
	## NOTE see src/main/unique.c in the R
	##	source code.
	k <- prod(D)
	if (k > 1073741824L)
	    stop("number of cells %d too large for hashing", k)
	i <- .Call(R_vector_index, D, I)
	I <- arrayInd(seq_len(k), .dim = D)
	k <- .Call(R_vector_index, D, I)
	i <- match(i, k)
	rm(k)
    } else {
	if (EXPAND < 3L) {
	    i <- which(D == -1L)
	    if (length(i)) {
		D[i]   <- 1L
		N[i]   <- list("1")
		I[, i] <- 1L
	    }
	}
	i <- .Call(R_match_matrix, I, NULL, NULL)
	I <- I[i[[2L]],, drop = FALSE]
	i <-   i[[1L]]
    }
    attributes(i) <-
	list(levels = seq_len(dim(I)[1L]),
	     class  = "factor")
    if (EXPAND == 1L) {
	V <- split.default(V, i)
	rm(i)
	names(V) <- NULL
	V <- lapply(V, FUN, ...)
    } else {
        verbose <- getOption("verbose")
	.pt <- proc.time()
        if(verbose)
            message(gettextf("processing %d cells ... ", dim(I)[1L]),
                    appendLF = FALSE,
                    domain = NA)
	i <- split.default(seq_along(i), i)
	names(i) <- NULL
	V <- mapply(function(i, z) {
		z <- I[z, ]
		z <- mapply(`[`, T, z)
		if (EXPAND > 2L) {
		    ## NOTE this consumes less computation time
		    ##	    and memory than
		    ## z <- array(vector(typeof(V),1L), z)
		    ## z[P[i,, drop = FALSE]] <- V[i]
		    z <- .Call(R_ini_array, z, P, V, i)
		    FUN(z, ...)
		} else
		    FUN(V[i], prod(z) - length(i), ...)
	    },
	    i,
	    seq_along(i),
	    SIMPLIFY = FALSE, USE.NAMES = FALSE
	)
	rm(i, T)
	if (EXPAND > 2L)
	    rm(P)
        if(verbose)
            message(sprintf("[%.2fs]\n", (proc.time() - .pt)[3L]),
                    appendLF = FALSE,
                    domain = NA)
    }
    if (all(unlist(lapply(V, length)) == 1L)) {
	V <- unlist(V, recursive = FALSE, use.names = FALSE)
	if (is.null(V)) 
	    V <- vector(MODE, 0L)
	i <- V == vector(typeof(V), 1L)
	i <- which(i)
	if (length(i)) {
	    i <- - i
	    I <- I[i,, drop = FALSE]
	    V <- V[i]
	}
    }
    x <- simple_sparse_array(I, V, D, N)
    rm(I, V, D, N)
    if (DROP)
	x <- drop_simple_sparse_array(x)
    x
}


rollup.simple_triplet_matrix <- 
function(x, MARGIN, INDEX = NULL, FUN = sum, ..., REDUCE = FALSE) {
    FUN <- match.fun(FUN)
    if (!identical(FUN, sum)) {
	if (!is.null(list(...)$DROP))
	    stop("'DROP' not supported")
	x <- rollup.simple_sparse_array(as.simple_sparse_array(x), 
	    MARGIN, INDEX, FUN, ...
	)
	return(as.simple_triplet_matrix(x))
    }
    if (is.character(MARGIN)) 
	MARGIN <- match(MARGIN, names(dimnames(x)))
    if (!all(match(MARGIN, seq_along(dim(x)), nomatch = 0L)))
	stop("'MARGIN' invalid")
    if (is.null(INDEX))
	INDEX <- vector("list", length(MARGIN))
    else {
	if (is.atomic(INDEX))
	    INDEX <- list(INDEX)
	if (length(INDEX) != length(MARGIN))
	    stop("'INDEX' invalid length")
    }
    names(INDEX) <- MARGIN
    for (k in MARGIN) {
	x <- switch(k,
	    t(rollup(t(x), 2L, INDEX[as.character(k)], FUN, ...)),
	    {
		z <- INDEX[[as.character(k)]]
		z <- 
		if (is.null(z))
		    rep.int(as.factor(1L), dim(x)[k])
		else {
		    if (length(z) != dim(x)[k])
			stop(gettextf("INDEX [%s] invalid length", k),
                             domain = NA)
		    as.factor(z)
		}
		.Call(R_row_tsums, 
		      x, z, 
		      if (is.null(list(...)$na.rm))
			  FALSE
		      else
			  as.logical(list(...)$na.rm), 
		      as.logical(REDUCE),
		      FALSE	## verbose
		)
	    }
	)
    }
    x
}

##
rollup.default <-
function(x, MARGIN, INDEX = NULL, FUN = sum, ..., DROP = FALSE, MODE = "double") {
    if (!length(dim(x)))
	stop("dim(x) must have a positive length")
    rollup(as.array(x), MARGIN, INDEX, FUN, ..., DROP = DROP, MODE = MODE)
}

###

library("slam")

x <- simple_triplet_diag_matrix(1, nrow = 3L)
rownames(x) <- letters[1:3]

identical(as.matrix(cbind(x, x)),
	  cbind(as.matrix(x), as.matrix(x)))
identical(as.matrix(rbind(t(x), t(x))),
	  rbind(as.matrix(t(x)), as.matrix(t(x))))

identical(as.matrix(cbind(x, t(x))),
	  cbind(as.matrix(x), as.matrix(t(x))))
identical(as.matrix(rbind(t(x), x)),
	  rbind(as.matrix(t(x)), as.matrix(x)))

###

library("slam")
set.seed(20110217)

###
x <- matrix(sample(c(0,1), 12, TRUE), ncol = 3L)
s <- as.simple_triplet_matrix(x)
s

## make invalid row indexes
s$i[sample(seq_along(s$i), 3)] <- 0L

try(row_sums(s), silent = FALSE)

###

##
library("slam")
x <- matrix(1:12, 4, dimnames = list(NULL, B = 1:3))
s <- as.simple_sparse_array(x)
s

extend_simple_sparse_array(s,  0L)
extend_simple_sparse_array(s, -1L)  ## the same
extend_simple_sparse_array(s,  1L)
extend_simple_sparse_array(s,  2L)
extend_simple_sparse_array(s, -3L)  ## the same

extend_simple_sparse_array(s, c( 0L,  0L))
extend_simple_sparse_array(s, c(-3L, -3L))

## automatic
z <- abind_simple_sparse_array(s, 1:3)
z
all.equal(as.array(z), rbind(x, 1:3))
z <- abind_simple_sparse_array(1:4, s, MARGIN = 2L)
z
all.equal(as.array(z), cbind(1:4, x))

abind_simple_sparse_array(1:3, array(2:4, c(1,3)), array(3:8, c(1,2,3)))
abind_simple_sparse_array(1:3, array(2:4, c(3,1)), array(3:8, c(3,2,1)), MARGIN = 3L) 

## manual
abind_simple_sparse_array(1:3, 2:4)
abind_simple_sparse_array(1:3, 2:4, MARGIN = -1L)
abind_simple_sparse_array(1:3, 2:4, MARGIN = -2L)

###

##
library(slam)

## zero dimension
a <- as.simple_sparse_array(array(0L, 0L))
drop_simple_sparse_array(a)

## invalid
a <- simple_sparse_array(c(1L, 2L), c(1L, -1L))
a$i[2L] <- 1L
a <- reduce_simple_sparse_array(a)
as.array(a)

## not minimal
x <- matrix(1:6, 3L, 2, dimnames = list(NULL, NULL))
a <- as.simple_sparse_array(x)
z <- reduce_simple_sparse_array(a)
identical(a, z)

##
v <- c("logical", "integer", "double", "complex", "character", "list")
stopifnot(any(sapply(v, function(v) 
    !.Call(slam:::R__valid_v, vector(typeof(v), 1L)))))

##

## A simple class for sparse (triplet) matrices.

## Mostly intended for being able to take advantage of LP solvers which
## allow for sparse specifictions of (possible rather large) constraint
## matrices.

simple_triplet_matrix <-
function(i, j, v, nrow = max(i), ncol = max(j), dimnames = NULL)
{
    stm <- list(i = as.integer(i), j = as.integer(j), v = v,
                nrow = as.integer(nrow), ncol = as.integer(ncol),
                dimnames = dimnames)
    if(anyDuplicated(.Call(R_list_of_index_pairs, stm$i, stm$j)) > 0)
        stop("Duplicate (i, j) pairs are not allowed.")
    class(stm) <- "simple_triplet_matrix"
    if(!.Call(R__valid_stm, stm))
	stop("failed to create a valid 'simple_triplet_matrix' object")
    stm
}

.is_sparse_mat_coercible_to_stm <-
function(x)
    UseMethod(".is_sparse_mat_coercible_to_stm")

.is_sparse_mat_coercible_to_stm.simple_triplet_matrix <-
function(x)
    TRUE

.is_sparse_mat_coercible_to_stm.default <-
function(x)
    FALSE

as.simple_triplet_matrix <-
function(x)
    UseMethod("as.simple_triplet_matrix")

as.simple_triplet_matrix.simple_triplet_matrix <- identity

as.simple_triplet_matrix.matrix <-
function(x)
{
    x <- unclass(x)
    if(!prod(dim(x)))
        return(simple_triplet_matrix(integer(), integer(), c(x),
                                     nrow = nrow(x), ncol = ncol(x),
                                     dimnames = dimnames(x)))
    ind <- which(is.na(x) | (x != vector(typeof(x), 1L)), arr.ind = TRUE)
    dimnames(ind) <- NULL
    simple_triplet_matrix(ind[, 1L], ind[, 2L], x[ind],
                          nrow = nrow(x), ncol = ncol(x),
                          dimnames = dimnames(x))
}

as.simple_triplet_matrix.default <-
function(x)
    as.simple_triplet_matrix(unclass(as.matrix(x)))

## Sparse matrix classes in package 'Matrix'.

as.simple_triplet_matrix.dgTMatrix <-
function(x)
{
    simple_triplet_matrix(x@i + 1L, x@j + 1L, x@x,
                          x@Dim[1L], x@Dim[2L], x@Dimnames)
}

as.simple_triplet_matrix.dgCMatrix <-
function(x)
{
    nc <- x@Dim[2L]
    simple_triplet_matrix(x@i + 1L, rep.int(seq_len(nc), diff(x@p)),
                          x@x,
                          x@Dim[1L], nc, x@Dimnames)
}

as.simple_triplet_matrix.dgRMatrix <-
function(x)
{
    nr <- x@Dim[1L]
    simple_triplet_matrix(rep.int(seq_len(nr), diff(x@p)), x@j + 1L,
                          x@x,
                          nr, x@Dim[2L], x@Dimnames)
}

.is_sparse_mat_coercible_to_stm.dgTMatrix <-
.is_sparse_mat_coercible_to_stm.dgCMatrix <-
.is_sparse_mat_coercible_to_stm.dgRMatrix <-
function(x)
    TRUE

## See work/Matrix.R for S4 methods for coercing simple triplet matrices
## to Matrix objects.

## Sparse matrix classes in package 'SparseM'.

as.simple_triplet_matrix.matrix.coo <-
function(x)
    simple_triplet_matrix(x@ia, x@ja, x@ra,
                          x@dimension[1L], x@dimension[2L])

as.simple_triplet_matrix.matrix.csc <-
function(x)
{
    nc <- x@dimension[2L]
    simple_triplet_matrix(x@ja, rep.int(seq_len(nc), diff(x@ia)), x@ra,
                          x@dimension[1L], nc)
}

as.simple_triplet_matrix.matrix.csr <-
function(x)
{
    nr <- x@dimension[1L]
    simple_triplet_matrix(rep.int(seq_len(nr), diff(x@ia)), x@ja, x@ra,
                          nr, x@dimension[2L])
}

.is_sparse_mat_coercible_to_stm.matrix.coo <-
.is_sparse_mat_coercible_to_stm.matrix.csc <-
.is_sparse_mat_coercible_to_stm.matrix.csr <-
function(x)
    TRUE
    
## Sparse matrix class in package 'spam'.

as.simple_triplet_matrix.spam <-
function(x)
{
    nr <- x@dimension[1L]
    simple_triplet_matrix(rep.int(seq_len(nr), diff(x@rowpointers)),
                          x@colindices, x@entries,
                          nr, x@dimension[2L])
}

.is_sparse_mat_coercible_to_stm.spam <-
function(x)
    TRUE

as.matrix.simple_triplet_matrix <-
function(x, ...)
{
    nr <- x$nrow
    nc <- x$ncol
    y <- matrix(vector(typeof(x$v), prod(nr, nc)), nr, nc)
    y[cbind(x$i, x$j)] <- x$v
    dimnames(y) <- x$dimnames
    y
}

as.array.simple_triplet_matrix <-
function(x, ...)
    as.array(as.matrix.simple_triplet_matrix(x, ...))

as.simple_triplet_matrix.simple_sparse_array <-
function(x) {
    dx <- x$dim
    if(length(dx) == 1L) {
        simple_triplet_matrix(
	    i = x$i[, 1L],
            j = rep.int(1L, nrow(x$i)),
            v = x$v,
            nrow = dx,
            ncol = 1L,
            dimnames = 
		if (!is.null(x$dimnames))
		    c(x$dimnames, list(NULL))
		else
		    NULL
	)
    } 
    else 
    if(length(dx) == 2L) {
	simple_triplet_matrix(
	    i = x$i[,1L],
	    j = x$i[,2L],
	    v = x$v,
	    nrow = x$dim[1L],
	    ncol = x$dim[2L],
	    dimnames = x$dimnames
	)
    }
    else
	stop("Unsupported number of dimensions")
}


is.simple_triplet_matrix <-
function(x)
    inherits(x, "simple_triplet_matrix")

is.numeric.simple_sparse_array <-
is.numeric.simple_triplet_matrix <-
function(x)
    is.numeric(x$v)

Math.simple_triplet_matrix <-
function(x, ...)
{
    ## Functions in the Math group mapping 0 to 0:
    funs <- c("abs", "sign", "sqrt",
              "floor", "ceiling", "trunc", "round", "signif")
    if(is.na(match(as.character(.Generic), funs)))
        stop(gettextf("Generic '%s' not defined for \"%s\" objects.",
                      .Generic, .Class),
             domain = NA)

    x$v <- get(.Generic)(x$v, ...)
    x
}

Ops.simple_triplet_matrix <-
function(e1, e2)
{
    ## Currently, we only implement the following (for numeric
    ## operands):

    ## * Unary plus and minus.
    ##
    ## * Addition, subtraction and multiplication of two compatible
    ##   simple triplet matrices (or operands coercible to these).
    ##   [Division by a simple triplet matrix typically involves
    ##   division by zero and hence is not provided.]
    ##
    ## * Multiplication and division of a simple triplet matrix x by a
    ##   number or a vector of length nrow(x) (allowing to conveniently
    ##   scale the rows of a numeric simple triplet matrix).
    ##
    ## * Non-equality comparison of a simple triplet matrix with 0.
    ##
    ## * Comparisons of the elements of a simple triplet matrix with a
    ##   number.
    ##
    ## More could be added (but note that the elements could have
    ## arbitrary modes).

    ## Drop zero-valued elements
    .reduce <- function(x) {
	ind <- which(!x$v)
	if(length(ind)) {
	    ind <- -ind
	    x$i <- x$i[ind]
	    x$j <- x$j[ind]
	    x$v <- x$v[ind]
	}
	x
    }

    op <- as.character(.Generic)

    if(nargs() == 1L) {
        if(op == "+") return(e1)
        if(op == "-") {
            e1$v <- - e1$v
            return(e1)
        }
        stop(gettextf("Unary '%s' not defined for \"%s\" objects.",
                      .Generic, .Class),
             domain = NA)
    }

    if(!(op %in% c("+", "-", "*", "/", "^",
                   "==", "!=", "<", "<=", ">", ">=")))
        stop(gettextf("Generic '%s' not defined for \"%s\" objects.",
                      .Generic, .Class),
             domain = NA)

    ## Require numeric operands for the arithmetic operators.
    if(!is.numeric(e1) || !is.numeric(e2))
        stop("Not implemented.")

    if(op %in% c("==", "!=", "<", "<=", ">", ">=")) {
        if(length(e2) == 1L) {
            if(is.na(e2))
                stop("NA/NaN handling not implemented.")
	    names(e2) <- NULL
            ind <- if(do.call(.Generic, list(0, e2))) {
                ## This inverts the sparse storage advantage, and hence
                ## will typically be inefficient.  Need to find the row
                ## and column positions of the zero entries.
                m <- matrix(TRUE, e1$nrow, e1$ncol)
                m[cbind(e1$i, e1$j)] <- FALSE
                which(m, arr.ind = TRUE)
            } else 
		integer()
            e1$v <- do.call(.Generic, list(e1$v, e2))
	    e1 <- .reduce(e1)
            if(n <- NROW(ind)) {
                e1$i <- c(e1$i, ind[, 1L])
                e1$j <- c(e1$j, ind[, 2L])
                e1$v <- c(e1$v, rep.int(TRUE, n))
            }
            return(e1)
        }
        stop("Not implemented.")
    }

    if(op == "^") {
        ## Allow for taking (single) positive exponents.
        if(is.object(e2) || (length(e2) != 1L) ||
           !is.finite(e2) || (e2 <= 0))
            stop("Not implemented.")
	names(e2) <- NULL
        e1$v <- e1$v ^ e2
        return(e1)
    }

    .make_dimnames <- function(e1, e2) {
        if(is.null(rnms <- rownames(e1)))
            rnms <- rownames(e2)
        if(is.null(cnms <- colnames(e1)))
            cnms <- colnames(e2)
        if(is.null(rnms) && is.null(cnms))
            NULL
        else {
            out <- list(rnms, cnms)
            if(is.null(nms <- names(dimnames(e1))))
                nms <- names(dimnames(e2))
            names(out) <- nms
            out
        }
    }

    ## Obviously, the following could be generalized ...

    if(op == "*") {
	if(!is.object(e1)) {
	    e3 <- e2
	    e2 <- e1
	    e1 <- e3 
	}
        if(!is.object(e2)) {
            if(length(e2) == 1L) {
		if(!is.finite(e2))
		    return(as.simple_triplet_matrix(as.matrix(e1) * e2))
		names(e2) <- NULL
		e1$v <- e1$v * e2
		if(!e2)
		    e1 <- .reduce(e1)
                return(e1)
            }
            if(length(e2) == e1$nrow) {
		names(e2) <- NULL
		pos <- which(!is.finite(e2))
		if(length(pos)) {
		    ## replace with dense rows
		    ind <- match(e1$i, pos, nomatch = 0L) == 0L
		    e1$v <- c(e1$v[ind],
                              as.matrix(e1[pos, ]))
		    e1$i <- c(e1$i[ind],
                              rep.int(pos, e1$ncol))
		    e1$j <- c(e1$j[ind],
                              rep(seq_len(e1$ncol), each = length(pos)))
		}
                e1$v <- e1$v * e2[e1$i]
		if(any(!e2))
		    e1 <- .reduce(e1)
                ## Could add something like
                ##    if(is.null(e1$dimnames) &&
                ##       !is.null(nms <- names(e2))) {
                ##        e1$dimnames <- list(nms, NULL)
                ##    }
                ## but then multiplying a matrix and a vector does not
                ## seem to do this either ...
                return(e1)
            }
	    if(is.matrix(e2)) {
		if(!all(dim(e2) == c(e1$nrow, e1$ncol)))
		    stop("Incompatible dimensions.")
		pos <- which(!is.finite(e2))
		if(length(pos)) {
		    ## add zeros
		    pos <- pos[match(pos, e1$i + (e1$j - 1L) * e1$nrow,
				     nomatch = 0L) == 0L] - 1L
		    if(length(pos)) {
			e1$v <- c(e1$v, vector(typeof(e1$v), length(pos)))
			e1$i <- c(e1$i, pos %%  e1$nrow + 1L)
			e1$j <- c(e1$j, pos %/% e1$nrow + 1L)
		    }
		}
		e1$v <- e1$v * e2[cbind(e1$i, e1$j)]
		if (any(!e2))
		    e1 <- .reduce(e1)
                e1$dimnames <- .make_dimnames(e1, e2)
		return(e1)
	    }
	    stop("Not implemented.")
        }
        ## This leaves multiplying two simple triplet matrices.
        e1 <- as.simple_triplet_matrix(e1)
        e2 <- as.simple_triplet_matrix(e2)
        ## Check dimensions: currently, no recycling.
        if(((nr <- e1$nrow) != e2$nrow) || ((nc <- e1$ncol) != e2$ncol))
            stop("Incompatible dimensions.")
        if(length(e1$v) < length(e2$v)) {
            ## Swap e1 and e2 so that duplicated indices can be found
            ## more efficiently.
            e3 <- e1
            e1 <- e2
            e2 <- e3
        }
        ## Find duplicated indices.
        ## pos <- match(paste(e2$i, e2$j, sep = "\r"),
        ##              paste(e1$i, e1$j, sep = "\r"),
        ##              nomatch = 0L)
        pos <- .Call(R_match_matrix, cbind(e1$i, e1$j),
                                     cbind(e2$i, e2$j), 0L)[[2L]]
        ind <- which(pos > 0L)
        if(!all(is.finite(e1$v)) || !all(is.finite(e2$v))) {
	    ## Augment and reduce
	    e2$i <- c(e2$i[ind], e2$i[-ind], e1$i[-pos])
	    e2$j <- c(e2$j[ind], e2$j[-ind], e1$j[-pos])
	    e2$v <- c(e2$v[ind] * e1$v[pos],
                      vector(typeof(e2$v), 1L) * c(e2$v[-ind], e1$v[-pos]))
	    e2$dimnames <- .make_dimnames(e1, e2)
	    return(.reduce(e2))
	} else
	    return(simple_triplet_matrix(e2$i[ind], e2$j[ind],
                                         e2$v[ind] * e1$v[pos],
                                         nr, nc, .make_dimnames(e1, e2)))
    }

    ## This is slightly inefficent but special value handling is already
    ## in place.  Note v / 0 = v * 0^(-1) = v * Inf.
    if(op == "/") {
        if(!is.object(e2))
	    return(e1 * e2^(-1))
	e2 <- as.matrix(e2)
	if (!is.object(e1))
	    return(as.simple_triplet_matrix(e1 * e2^(-1)))
	return(e1 * e2^(-1))
    }

    ## This leaves adding and subtracting two simple triplet matrices.
    e1 <- as.simple_triplet_matrix(e1)
    e2 <- if(op == "+")
        as.simple_triplet_matrix(e2)
    else
        as.simple_triplet_matrix(-e2)
    ## Check dimensions: currently, no recycling.
    if((e1$nrow != e2$nrow) || (e1$ncol != e2$ncol))
        stop("Incompatible dimensions.")
    if(length(e1$v) < length(e2$v)) {
        ## Swap e1 and e2 so that duplicated indices can be found more
        ## efficiently.
        e3 <- e1
        e1 <- e2
        e2 <- e3
    }
    ## Find duplicated indices.
    ## pos <- match(paste(e2$i, e2$j, sep = "\r"),
    ##              paste(e1$i, e1$j, sep = "\r"),
    ##              nomatch = 0L)
    pos <- .Call(R_match_matrix, cbind(e1$i, e1$j),
                                 cbind(e2$i, e2$j), 0L)[[2L]]
    ind <- which(pos == 0L)
    ## Notice 0 + special value = special value.
    e1$v[pos] <- e1$v[pos] + e2$v[pos > 0L]
    e1$i <- c(e1$i, e2$i[ind])
    e1$j <- c(e1$j, e2$j[ind])
    e1$v <- c(e1$v, e2$v[ind])
    e1$dimnames <- .make_dimnames(e1, e2)
    .reduce(e1)
}

Summary.simple_triplet_matrix <-
function(..., na.rm = FALSE)
{
    v <- unlist(lapply(list(...),
                       function(e) {
                           v <- as.simple_triplet_matrix(e)$v
                           if(length(v) < prod(dim(e)))
                               v <- c(v, vector(typeof(v), 1L))
                           v
                       }),
                recursive = FALSE)
    do.call(.Generic, list(v, na.rm = na.rm))
}

dim.simple_triplet_matrix <-
function(x)
    c(x$nrow, x$ncol)

`dim<-.simple_triplet_matrix` <-
function(x, value)
{
    value <- as.integer(value)
    if((length(value) != 2L) || any(is.na(value)))
        stop("invalid dim replacement value")
    nr <- x$nrow
    nc <- x$ncol
    if(prod(value) != prod(nr, nc))
        stop("invalid dim replacement value")

    pos <- nr * (x$j - 1L) + x$i - 1L

    nr <- value[1L]
    nc <- value[2L]

    x$i <- pos %%  nr + 1L
    x$j <- pos %/% nr + 1L
    x$nrow <- nr
    x$ncol <- nc
    x$dimnames <- NULL

    x
}

dimnames.simple_triplet_matrix <-
function(x)
    x$dimnames

`dimnames<-.simple_sparse_array` <-
`dimnames<-.simple_triplet_matrix` <- 
function(x, value)
{
    if(!is.null(value)) {
        ## NOTE that if length(value) < length(dim(x)) we
	##      have to assume that the dimensions with index
	##      seq_len(length(value)) are to be set. For 
	##	example, we are called with a list of length
	##      one if we call dimnames(x)[[1L]] <- value and
	##      dimnames(x) == NULL (because of [[<-)
	##
        if(!is.list(value) || length(value) > length(dim(x)))
            stop("Invalid dimnames.")
        if(!length(value))
            value <- NULL
        else {
            dnx <- vector("list", length(dim(x)))
	    len <- lengths(value)
	    ind <- which(len > 0L)
	    if (any(len[ind] != dim(x)[ind]))
		stop("Invalid component length.")
            dnx[ind] <- lapply(value[ind], as.character)
	    if (!is.null(names(value))) {
		ind <- seq_len(length(value))
		names(dnx)[ind] <- names(value)
	    }
        }
    }
    ## See the constructor (above).
    if(is.null(value))
        x$dimnames <- NULL
    else
        x$dimnames <- dnx
    x
}

## For reuse in other functions (we want to mess up, too).
.stm_as_subscript <- 
function(x, d, safe = FALSE, ...) 
{
    if (!is.simple_triplet_matrix(x))
	return(x)
    if (!is.logical(x$v) || !identical(dim(x), d))
	stop("Not implemented.")
    ## need column-major order
    k <- order(x$j, x$i)
    if (any(diff(k < 0))) {
	x$v <- x$v[k]
	x$i <- x$i[k]
	x$j <- x$j[k]
    }
    ## offer a choice
    if (safe ||
	log2(prod(dim(x))) > .Machine$double.digits)
	cbind(x$i[x$v], x$j[x$v])
    else
	## need to use a double in expression to 
	## get a result of type double
	((x$j - 1) * x$nrow + x$i)[x$v]
}

`[.simple_triplet_matrix` <-
function(x, i, j, drop = FALSE)
{
    ## (Well, we certainly don't drop ...)

    ## (See e.g. `[.data.frame` for the trickeries of subscript methods:
    ## e.g., 
    ##   x[i = sample.int(nr, k), , drop = FALSE]
    ## counts 4 arguments (x, i, j and drop) where j is missing ...

    na <- nargs() - !missing(drop)
    if((na == 1L) ||
       (na == 2L) && missing(i) ||
       (na == 3L) && missing(i) && missing(j))
        return(x)
    
    nr <- x$nrow
    nc <- x$ncol

    pd <- prod(nr, nc)

    ## FIXME eventually, we should get rid of ill-conceived features 
    ##       which need to expand to dense.
    .disable <- pd > slam_options("max_dense")

    if(na == 2L) {
        ## Single index subscripting.
        ## Mimic subscripting matrices: no named argument handling in
        ## this case.

	## FIXME mapping to numeric seems to be less inefficient.
	i <- .stm_as_subscript(i, c(nr, nc))
        if(is.character(i))
            out <- vector(typeof(x$v))[rep.int(NA, length(i))]
        else if(!is.matrix(i)) {
            if(is.logical(i)) {
		if(.disable)
		  stop("Logical vector subscripting disabled for this object.")
                i <- which(rep_len(i, pd))
	    }
            else if(!is.numeric(unclass(i)))
                stop(gettextf("Invalid subscript type: %s.",
                              typeof(i)),
                     domain = NA)
	    else
		if(log2(pd) > .Machine$double.digits)
		  stop("Numeric vector subscripting disabled for this object.")
	    ## Shortcut
	    if(!length(i))
		return(vector(mode = typeof(x$v), length = 0L))
	    if(is.double(i))
		i <- trunc(i)
            ## Let's hope we have a vector.
            ## What if we have both negatives and positives?
            if(all(i >= 0, na.rm = TRUE)) {
                i <- i[i > 0]
                out <- vector(mode = typeof(x$v), length = length(i))
		if(length(out)) {
		    is.na(i) <- i > pd
		    is.na(out) <- is.na(i)
		    i <- match(i, (x$j - 1) * nr + x$i, 0L)
		    out[i > 0L] <- x$v[i]
		}
            } else if(!any(is.na(i)) && all(i <= 0)) {
		if(.disable)
		  stop("Negative vector subscripting disabled for this object.")
                out <- vector(mode = typeof(x$v), pd)
                out[(x$j - 1L) * nr + x$i] <- x$v
                out <- out[i]
            }
            else stop("Cannot mix positive and negative subscripts.")
        }
        else {
	    ## Shortcut
	    if(!nrow(i))
		return(vector(mode = typeof(x$v), length = 0L))
	    ## Ignore dimensions
	    if(ncol(i) != 2L || !is.numeric(i))
		return(do.call(`[.simple_triplet_matrix`,
			       list(x = x, as.vector(i))))

	    if(is.double(i))
		i <- trunc(i)
            ## Rows containing zero indices can be dropped.
            ## Rows with NA indices should give NA (at least for
            ## non-recursive x).
	    k <- .Call(R_all_row, i > 0, FALSE)
            i <- i[k, ,drop = FALSE]
            ## Note that negative values are not allowed in a matrix
            ## subscript.
            if(any(i < 0, na.rm = TRUE))
                stop("Invalid subscript.")
            out <- vector(mode = typeof(x$v), length = nrow(i))
	    if(length(out)) {
		if (any(i > rep(c(nr, nc), each = nrow(i)), na.rm = TRUE))
		    stop("subscript out of bounds")
		k <- k[k]
		is.na(out) <- is.na(k)
		rm(k)
		## See duplicated.matrix
		## pos <- match(paste(i[, 1L], i[, 2L], sep = "\r"),
		##	     paste(x$i, x$j, sep = "\r"),
		##	     nomatch = 0L)
		storage.mode(i) <- "integer"
		i <- .Call(R_match_matrix, cbind(x$i, x$j), i, 0L)[[2L]]
		out[i > 0L] <- x$v[i]
	    }
        }
    }
    else {
        ## Two index subscripting is rather tricky, as it can also be
        ## used for rearranging and "recycling" rows and columns.  Let
        ## us not support the latter for now, so that selected rows and
        ## columns must be unique.
        pos <- NULL
        if(!missing(i)) {
	    if(any(is.na(i)))
		stop("NA indices not allowed.")
            pi <- seq_len(nr)
            if(is.logical(i)) {
                i <- rep_len(i, nr)
                nr <- sum(i)
                pos <- i[x$i]
            } else {
                if(is.character(i)) {
                    i <- match(i, rownames(x))
                    if(any(is.na(i)))
                        stop("Subscript out of bounds.")
                    if(any(duplicated(i)))
                        stop("Repeated indices currently not allowed.")
                } else if(is.numeric(i)) {
		    if(is.double(i))
			i <- trunc(i)
                    if(all(i >= 0)) {
                        i <- i[i > 0]
			if(any(i > nr))
			    stop("subscript out of bounds")
                        if(any(duplicated(i)))
                            stop("Repeated indices currently not allowed.")
                    } else if(all(i <= 0))
                        i <- pi[i]
                    else
                        stop("Cannot mix positive and negative subscripts.")
                } else {
                    stop(gettextf("Invalid subscript type: %s.",
                                  typeof(i)),
                         domain = NA)
                }
                nr <- length(i)
                pos <- match(x$i, i, 0L) > 0L
            }
            pi[i] <- seq_len(nr)
        }
        if(!missing(j)) {
	    if(any(is.na(j)))
		stop("NA indices not allowed.")
            pj <- seq_len(nc)
            if(is.logical(j)) {
                j <- rep_len(j, nc)
                nc <- sum(j)
                pos <- if(is.null(pos))
                    j[x$j]
                else
                    j[x$j] & pos
            } else {
                if(is.character(j)) {
                    j <- match(j, colnames(x))
                    if(any(is.na(j)))
                        stop("Subscript out of bounds.")
                    if(any(duplicated(j)))
                        stop("Repeated indices currently not allowed.")
                } else if(is.numeric(j)) {
		    if(is.double(j))
			j <- trunc(j)
                    if(all(j >= 0)) {
                        j <- j[j > 0]
			if(any(j > nc))
			    stop("subscript out of bounds")
                        if(any(duplicated(j)))
                            stop("Repeated indices currently not allowed.")
                    } else if(all(j <= 0))
                        j <- pj[j]
                    else
                        stop("Cannot mix positive and negative subscripts.")
                } else {
                    stop(gettextf("Invalid subscript type: %s.",
                                  typeof(j)),
                         domain = NA)
                }
                nc <- length(j)
                pos <- if(is.null(pos))
                    (match(x$j, j, 0L) > 0L)
                else
                    (match(x$j, j, 0L) > 0L) & pos
            }
            pj[j] <- seq_len(nc)
        }

        if(!is.null(dnx <- x$dimnames)) {
	    if (!missing(i)) {
		dnx[1L] <- list(dnx[[1L]][i])
		if (!length(dnx[[1L]]))
		    dnx[1L] <- list(NULL)
	    }
	    if (!missing(j)) {
		dnx[2L] <- list(dnx[[2L]][j])
		if (!length(dnx[[2L]]))
		    dnx[2L] <- list(NULL)
	    }
	    if (!length(dnx[[1L]]) && !length(dnx[[2L]]))
		dnx <- NULL
	}

        i <- if(missing(i)) x$i[pos] else pi[x$i[pos]]
        j <- if(missing(j)) x$j[pos] else pj[x$j[pos]]
        out <- simple_triplet_matrix(i, j, x$v[pos], nr, nc, dnx)
    }

    out
}

rbind.simple_triplet_matrix <-
function(..., deparse.level = 1L)
{
    args <- lapply(Filter(Negate(is.null), list(...)),
                   as.simple_triplet_matrix)
    ## Ignore 'deparse.level' ...
    out <- Reduce(function(x, y) {
        if((nc <- ncol(x)) != ncol(y))
            stop("Numbers of columns of matrices must match.")
        nr <- nrow(x)
        simple_triplet_matrix(c(x$i, y$i + nr),
                              c(x$j, y$j),
                              c(x$v, y$v),
                              nrow = nr + nrow(y), ncol = nc)
    }, args)
    ## Handle dimnames in one final step.
    rnms <- lapply(args, rownames)
    rnms <- if(!all(vapply(rnms, is.null, NA))) {
	rnms <- mapply(function(rnm, n)
	    if(is.null(rnm))
		rep.int("", n)
	    else
		rnm,
	    rnms,
	    lapply(args, nrow),
	    SIMPLIFY = FALSE
	)
	
        do.call(c, rnms)
    }
    else
        NULL
    cnms <- Find(Negate(is.null), lapply(args, colnames))
    dimnames(out) <- list(rnms, cnms)
    out
}

cbind.simple_triplet_matrix <-
function(..., deparse.level = 1L)
{
    args <- lapply(Filter(Negate(is.null), list(...)),
                   as.simple_triplet_matrix)
    ## Ignore 'deparse.level' ...
    out <- Reduce(function(x, y) {
        if((nr <- nrow(x)) != nrow(y))
            stop("Numbers of rows of matrices must match.")
        nc <- ncol(x)
        simple_triplet_matrix(c(x$i, y$i),
                              c(x$j, y$j + nc),
                              c(x$v, y$v),
                              nrow = nr, ncol = nc + ncol(y))
    }, args)
    ## Handle dimnames in one final step.
    cnms <- lapply(args, colnames)
    cnms <- if(!all(vapply(cnms, is.null, NA))) {
	cnms <- mapply(function(cnm, n)
	    if(is.null(cnm))
		rep.int("", n)
	    else
		cnm,
	    cnms,
	    lapply(args, ncol),
	    SIMPLIFY = FALSE
	)
        do.call(c, cnms)
    }
    else
        NULL
    rnms <- Find(Negate(is.null), lapply(args, rownames))
    dimnames(out) <- list(rnms, cnms)
    out
}

t.simple_triplet_matrix <-
function(x)
    simple_triplet_matrix(x$j, x$i, x$v, x$ncol, x$nrow, rev(x$dimnames))

duplicated.simple_triplet_matrix <-
function(x, incomparables = FALSE, MARGIN = 1L, fromLast = FALSE, ...)
{
    ## We could use the duplicated method for class matrix, but at the
    ## expense of going from sparse to dense ...
    if(!is.logical(incomparables) || incomparables)
        .NotYetUsed("incomparables != FALSE")
    if(MARGIN == 1L) {
        i <- x$i
        j <- x$j
        len <- x$nrow
    } else if(MARGIN == 2L) {
        i <- x$j
        j <- x$i
        len <- x$ncol
    } else
        stop("Invalid margin.")
    o <- order(i, j)
    y <- split(paste(j[o], x$v[o], sep = "\r"), i[o])
    tmp <- character(len)
    names(tmp) <- seq_along(tmp)
    tmp[names(y)] <- vapply(y, paste, "", collapse = "\r")
    duplicated(tmp, fromLast = fromLast)
}

unique.simple_triplet_matrix <-
function(x, incomparables = FALSE, MARGIN = 1L, fromLast = FALSE, ...)
{
    if(!is.logical(incomparables) || incomparables)
        .NotYetUsed("incomparables != FALSE")
    ind <- !duplicated(x, MARGIN = MARGIN, fromLast = fromLast)
    if(MARGIN == 1L)
        x[which(ind), ]
    else
        x[, which(ind)]
}

c.simple_triplet_matrix <-
function(..., recursive = FALSE)
{
    args <- list(...)
    ind <- which(vapply(args, inherits, NA, "simple_triplet_matrix"))
    args[ind] <-
        lapply(args[ind],
               function(x) {
                   y <- vector(typeof(x$v), prod(x$nrow, x$ncol))
                   y[x$i + (x$j - 1L) * x$nrow] <- x$v
                   y
               })
    do.call(c, args)
}

print.simple_triplet_matrix <-
function(x, ...)
{
    writeLines(gettextf("A %s simple triplet matrix.",
                        paste(dim(x), collapse = "x")))
    invisible(x)
}

mean.simple_triplet_matrix <-
function(x, ...)
{
    sum(x$v) / prod(dim(x))
}

aperm.simple_triplet_matrix <-
function(a, perm = NULL, ...)
{
    s <- c(1L, 2L)
    if(!is.null(perm)) {
        perm <- if(is.character(perm))
            match(perm, names(a$dimnames))
        else if(is.numeric(perm))
            match(perm, s)
        else NULL
        if(length(perm) != length(s) || any(is.na(perm)))
            stop("Invalid permutation.")
        if(all(perm == s))
            return(a)
    }
    ## Transpose.
    t.simple_triplet_matrix(a)
}

as.vector.simple_triplet_matrix <-
function(x, mode = "any")
    as.vector(as.matrix(x), mode)

split.simple_triplet_matrix <- 
function(x, f, drop = FALSE, MARGIN = 1L, ...)
{
    if(!is.factor(f))
        f <- as.factor(f)
    else if(drop)
        f <- factor(f)
        
    if (length(MARGIN) != 1L ||
	is.na(match(MARGIN, 1:2)))
	stop("'MARGIN' invalid")
    if (length(f) != dim(x)[MARGIN])
	stop("'f' invalid length")

    fx <- f[x[[MARGIN]]]
    mapply(function(i, j, v, k) {
	    z <- x
	    z$i <- i
	    z$j <- j
	    z$v <- v
	    z[[MARGIN]] <- match(z[[MARGIN]], k)
	    z[[MARGIN + 3L]] <- length(k)
	    k <- z$dimnames[[MARGIN]][k]
	    if (!is.null(k))
		z$dimnames[[MARGIN]] <- k
	    if (!.Call(R__valid_stm, z))
		stop("oops, invalid 'simple_triplet_matrix' object")
	    z
	},
	split(x$i, fx),
	split(x$j, fx),
	split(x$v, fx),
	split(seq_along(f), f),
	SIMPLIFY = FALSE
    )
}

## Utilities for creating special simple triplet matrices:

simple_triplet_zero_matrix <-
function(nrow, ncol = nrow, mode = "double")
    simple_triplet_matrix(integer(), integer(), vector(mode, 0L),
                          nrow, ncol)

simple_triplet_diag_matrix <-
function(v, nrow = length(v))
{
    v <- rep_len(v, nrow)
    i <- seq_len(nrow)
    simple_triplet_matrix(i, i, v, nrow, nrow)
}

library("slam")

m <- matrix(c(2, 5, 0, 8, 0,
              0, 0, 0, 0, 0,
              0, 0, 3, 2, 1),
            nr = 3, nc = 5, byrow = TRUE,
            dimnames = list(c("X", "Y", "Z"),
                            LETTERS[1 : 5]))
x <- as.simple_triplet_matrix(m)

identical(
    lapply(split(x, c(1, 2, 2)), as.matrix),
	   split.data.frame(m, c(1, 2, 2))
)
identical(
    lapply(split(x, c(1, NA, 2)), as.matrix),
	   split.data.frame(m, c(1, NA, 2))
)

local({
    colnames(x) <- NULL
    x[] <- 0
    identical(
	lapply(split(x, c(1, 1, 2, 2, 3), MARGIN = 2), as.matrix),
	lapply(split.data.frame(as.matrix(t(x)), c(1, 1, 2, 2, 3)), t)
    )
})

## CB 2012/9 2016/8
##
## FIXME extending might be useful unless implemented
##	 as for dense arrays.
##
`[<-.simple_sparse_array` <- 
function(x, ..., value) {

    if (inherits(value, c("simple_sparse_array", "simple_triplet_matrix"))) {
	if (prod(dim(value)) > slam_options("max_dense"))
	    stop("Replacement disabled.")
	value <- as.vector(value)
    }
    
    if (!length(value))
	stop("replacement has length zero")

    nd <- length(x$dim)
    pd <- prod(x$dim)

    .disable <- pd > slam_options("max_dense")

    na <- nargs()
    if (na == 3L && missing(..1))
	if (.disable)
	    stop("Empty subscripting disabled.")
	else
	    return(
		`[<-.simple_sparse_array`(x, seq_len(pd), value = value)
	    )
	    
    ## Single index subscripting.
    if (na == 3L) {
	I <- ..1
	## NOTE mapping to matrix is less inefficient (see below).
	I <- .stm_as_subscript(I, x$dim, TRUE)
	if (!is.numeric(unclass(I)))
	    stop("Only numeric / matrix subscripting is implemented.")
	if (!length(I))
	    return(x)
	## Missing values in subscripts.
	k <- is.na(I)
	if (any(k))
	    if (length(value) == 1L)
		I[k] <- 0L
	    else
		stop("NAs are not allowed in subscripted assignments")
	rm(k)
	## Vector subscripting.
	if (!is.matrix(I)) {
	    if (log2(pd) > .Machine$double.digits)
		stop("Vector subscripting disabled for this object.")
	    ## Map.
	    if (is.double(I))
		I <- trunc(I)
	    if (all(I >= 0L)) {
		## Remove zero subscripts.
		I <- I[I > 0L]
		if (!length(I))
		    return(x)
		if (any(I > pd))
		    stop("Extending is not implemented.")
	    } else {
		if (.disable)
		    stop("Negative subscripting disabled for this object.")
		if (all(I <= 0L)) {
		    ## NOTE this fails if NAs are introduced by 
		    ##	    coercion. 
		    I <- seq_len(pd)[I]
		} else
		    stop("only 0's may be mixed with negative subscripts")
	    }
	    ## Expand.
	    I <- arrayInd(I, .dim = x$dim)
	} else
	    ## NOTE as the other replacement rules are no less 
	    ##	    confusing we allow this, too.
	    if (ncol(I) != nd) {
		dim(I) <- NULL
		return(
		    `[<-.simple_sparse_array`(x, I, value = value)
		)
	    }
	    ## Map.
	    if (is.double(I))
		I <- trunc(I)
	    if (any(I < 0L))
		stop("negative values are not allowed in a matrix subscript")
	    ## Remove rows with zero subscripts.
	    I <- I[.Call(R_all_row, I > 0L, FALSE),, drop = FALSE]
	    if (!nrow(I))
		return(x)
	    ## NOTE NAs cannot be introduced by coercion as
	    ##      long as the bounds are integer.
	    if (any(I > rep(x$dim, each = nrow(I))))
		stop("subscript out of bounds")
	    storage.mode(I) <- "integer"
    } else {
	if (na != nd + 2L)
	    stop("incorrect number of dimensions")
        ## Get indices. 
        args <- vector("list", na - 2L)
	for (k in seq_along(args)) {
	    n <- as.name(sprintf("..%i", k))
	    if (!do.call(missing, list(n)))
		args[[k]] <- eval(n)
	    else
		if (.disable)
		    stop("Missing dimensions disabled for this object.")
		else
		    args[[k]] <- seq_len(x$dim[k])
	}
	if (!all(vapply(args, is.numeric, NA)))
	    stop("Only numeric subscripting is implemented.")
	## Replace negative subscripts.
	for (k in seq_along(args)) {
	    ## Map.
	    if (is.double(args[[k]]))
		args[[k]] <- trunc(args[[k]]) 
	    if (.disable) {
		if (any(args[[k]] < 0L))
		    stop("Negative subscripting disabled for this object.")
	    } else
		if (all(args[[k]] <= 0L))
		    args[[k]] <- seq_len(x$dim[k])[args[[k]]]
		else
		    if (!all(args[[k]] >= 0L))
			stop("only 0's may be mixed with negative subscripts")
	}
	## Expand.
	args <- matrix(
	    unlist(expand.grid(args), use.names = FALSE),
	    ncol = length(args)
	)
	return(
	    `[<-.simple_sparse_array`(x, args, value = value)
	)
    }

    ## Recycling.
    if (nrow(I) %% length(value))
	warning("number of items to replace is not a multiple of replacement length")
    V <- rep_len(value, nrow(I))

    ## Merge. 
    ##
    ## Emulates subsequent assignments of a sequence
    ## of replacement values with duplicate cell 
    ## indexes.
    I <- rbind(x$i, I)
    k <- .Call(R_match_matrix, I, NULL, NULL)
    k <- !duplicated(k[[1L]], fromLast = TRUE)
    I <- I[k,, drop = FALSE]
    V <- c(x$v, V)[k]

    ## Remove ZERO entries.
    k <- which(V == vector(typeof(V), 1L))
    if (length(k)) {
	k <- -k
	I <- I[k,, drop = FALSE]
	V <- V[k]
    }

    simple_sparse_array(
	v = V,
	i = I,
	dim = x$dim,
	dimnames = x$dimnames
    )
}

##
`[<-.simple_triplet_matrix` <- 
function(x, ..., value) {
    x <- `[<-.simple_sparse_array`(as.simple_sparse_array(x), ..., 
	value = value)
    if (inherits(x, "simple_sparse_array"))
	x <- as.simple_triplet_matrix(x)
    x
}

###

##
require("slam")

##
x <- matrix(c(1L, 0L, 3L, 0L, 5L, 0L), ncol = 2,
    dimnames = list(1:3, LETTERS[1:2]))
x
s <- as.simple_triplet_matrix(x)

colapply_simple_triplet_matrix(s, identity)
rowapply_simple_triplet_matrix(s, identity)

s$v <- as.numeric(s$v)
simplify2array(colapply_simple_triplet_matrix(s, identity))

s$v <- as.complex(s$v)
simplify2array(colapply_simple_triplet_matrix(s, identity))

s$v <- as.list(s$v)
simplify2array(colapply_simple_triplet_matrix(s, identity))

s$v <- as.character(s$v)
simplify2array(colapply_simple_triplet_matrix(s, identity))

##

## CB 2009/5,6,10 2010/6 2013/10


## NOTE the C code does not use long double for accumulation.
.means_simple_triplet_matrix <-
function(x, DIM, na.rm)
{
    s <- .Call(R_sums_stm, x, DIM, na.rm)
    n <- c(x$nrow, x$ncol)[-DIM]
    if (na.rm) {
	x$v <- is.na(x$v)
	nna <- .Call(R_sums_stm, x, DIM, FALSE)
	s / (n - nna)
    }
    else
	s /  n
}


## R interfaces

row_sums <-
function(x, na.rm = FALSE, dims = 1, ...)
    UseMethod("row_sums")

row_sums.default <-
function(x, na.rm = FALSE, dims = 1, ...)
    base::rowSums(x, na.rm, dims, ...)

row_sums.simple_triplet_matrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    .Call(R_sums_stm, x, 1L, na.rm)

row_sums.dgCMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::rowSums(x, na.rm = na.rm, dims = dims, ...)
row_sums.dgTMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::rowSums(x, na.rm = na.rm, dims = dims, ...)

col_sums <-
function(x, na.rm = FALSE, dims = 1, ...)
    UseMethod("col_sums")

col_sums.default <-
function(x, na.rm = FALSE, dims = 1, ...)
    base::colSums(x, na.rm, dims, ...)

col_sums.simple_triplet_matrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    .Call(R_sums_stm, x, 2L, na.rm)

col_sums.dgCMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::colSums(x, na.rm = na.rm, dims = dims, ...)
col_sums.dgTMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::colSums(x, na.rm = na.rm, dims = dims, ...)

row_means <-
function(x, na.rm = FALSE, dims = 1, ...)
    UseMethod("row_means")

row_means.default <-
function(x, na.rm = FALSE, dims = 1, ...)
    base::rowMeans(x, na.rm, dims, ...)

row_means.simple_triplet_matrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    .means_simple_triplet_matrix(x, DIM = 1L, na.rm)

row_means.dgCMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::rowMeans(x, na.rm = na.rm, dims = dims, ...)
row_means.dgTMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::rowMeans(x, na.rm = na.rm, dims = dims, ...)

col_means <-
function(x, na.rm = FALSE, dims = 1, ...)
    UseMethod("col_means")

col_means.default <-
function(x, na.rm = FALSE, dims = 1, ...)
    base::colMeans(x, na.rm, dims, ...)

col_means.simple_triplet_matrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    .means_simple_triplet_matrix(x, DIM = 2L, na.rm)

col_means.dgCMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::colMeans(x, na.rm = na.rm, dims = dims, ...)
col_means.dgTMatrix <-
function(x, na.rm = FALSE, dims = 1, ...)
    Matrix::colMeans(x, na.rm = na.rm, dims = dims, ...)

row_norms <-
function(x, p = 2)
{
    if(p == 2)
        sqrt(row_sums(x ^ 2))
    else if(p == 1)
        row_sums(abs(x))
    else if(p == Inf)
        c(rollup(abs(x), 2L, FUN = max))
    else
        row_sums(abs(x) ^ p) ^ (1/p)
}

col_norms <-
function(x, p = 2)
{
    if(p == 2)
        sqrt(col_sums(x ^ 2))
    else if(p == 1)
        col_sums(abs(x))
    else if(p == Inf)
        c(rollup(abs(x), 1L, FUN = max))
    else
        col_sums(abs(x) ^ p) ^ (1/p)
}

##
.nnzero <- 
function(x, scale = FALSE) {
    v <- c("simple_triplet_matrix", "simple_sparse_array")
    if (inherits(x, v))
	v <- x$v
    else {
	x <- as.array(x)
	v <- x
    }
    v <- v == vector(typeof(v), 1L)
    v <- v + 1L
    n <- length(v)
    v <- tabulate(v, 2L)
    v <- c(v, n - sum(v))
    names(v) <- c("nnzero", "nzero", NA)
    if (scale)
	v <- v / prod(dim(x))
    v
}

###

##
library("slam")

##
x <- matrix(c(1, 0, 0, 2, 1, NA), nrow = 2,
            dimnames = list(A = 1:2, B = 1:3))
x

a <- as.simple_sparse_array(x)
a

##
z <- rollup(x, 2L, c(1,2,1), na.rm = TRUE)
z
identical(as.array(z),
	  as.array(rollup(a, 2L, c(1,2,1), na.rm = TRUE)))
identical(as.array(z),
	  as.array(rollup(a, 2L, c(1,2,1), na.rm = TRUE, EXPAND = "dense")))
identical(as.array(z),
	  as.array(rollup(a, 2L, c(1,2,1), na.rm = TRUE, EXPAND = "all")))

##
z <- rollup(x, 2L, c(1,NA,1), na.rm = TRUE)
z
identical(as.array(z),
          as.array(rollup(a, 2L, c(1,NA,1), na.rm = TRUE)))
identical(as.array(z),
          as.array(rollup(a, 2L, c(1,NA,1), na.rm = TRUE, EXPAND = "dense")))
identical(as.array(z),
          as.array(rollup(a, 2L, c(1,NA,1), na.rm = TRUE, EXPAND = "all")))

##
z <- rollup(x, 2L, c(1,NA,1), na.rm = TRUE, DROP = TRUE)
identical(as.array(z),
          as.array(rollup(a, 2L, c(1,NA,1), na.rm = TRUE, DROP = TRUE)))


##
z <- rollup(x, 1:2, list(1:2, c(1,2,1)), na.rm = TRUE)
identical(as.array(z),
	  as.array(rollup(a, 1:2, list(1:2, c(1,2,1)), na.rm = TRUE)))

##
s <- as.simple_triplet_matrix(a)
z <- rollup(x, 2L, FUN = min, na.rm = TRUE)
identical(as.matrix(z),
	  as.matrix(rollup(s, 2L, FUN = min, na.rm = TRUE, EXPAND = "dense")))


## 2016/6
s <- simple_sparse_zero_array(dim = c(2, 3, 4))
z <- rollup(s, 1:2)
identical(as.array(z), 
	  rollup(as.array(s), 1:2))

###

## 
extend_simple_sparse_array <-
function(x, MARGIN = 0L)
{
    if (!is.simple_sparse_array(x))
	stop("'x' not of class 'simple_sparse_array'")
    k <- MARGIN < 0L
    MARGIN[k] <- -MARGIN[k] -1L
    k <- MARGIN[1L]
    ## extend
    D <- c(1L, x$dim)
    I <- cbind(1L, x$i)
    N <- x$dimnames
    if (!is.null(N))
	N <- c(list(NULL), N)
    if (k > 0L)
	if (k > length(D))
	    stop("'MARGIN' invalid")
	else {	## order
	    i <- order(c(k + 1L, seq.int(length(D) - 1L))) 
	    D <- D[i]
	    I <- I[,i]
	    if (!is.null(N))
		N <- N[i]
	}
    x <- simple_sparse_array(I, x$v, D, N)
    rm(I, D, N)
    while (length(MARGIN <- MARGIN[-1L])) {
	k <- MARGIN > k
	MARGIN[k] <- MARGIN[k] + 1L
	x <- extend_simple_sparse_array(x, MARGIN[[1L]])
    }
    x
}

##
abind_simple_sparse_array <-
function(..., MARGIN = 1L)
{
    if (length(MARGIN) != 1L || 
	MARGIN == 0L)
	stop("'MARGIN' invalid")
    args <- list(...)
    if (length(args))
	args <- args[!vapply(args, is.null, NA)]
    if (!length(args))
	return(NULL)
    x <- as.simple_sparse_array(args[[1L]])
    if (MARGIN < 0L)
	x <- extend_simple_sparse_array(x, MARGIN)
    if (length(args) == 1L)
	return(x)
    for (y in args[-1L]) {
	y <- as.simple_sparse_array(y)
	if (MARGIN < 0L)
	    y <- extend_simple_sparse_array(y, MARGIN)
	m <- abs(MARGIN)
	if (length(y$dim) == length(x$dim) - 1L)
	    y <- extend_simple_sparse_array(y, -min(m, length(x$dim)))
	else
	    if (length(y$dim) - 1L == length(x$dim)) {
		x <- extend_simple_sparse_array(x, -min(m, length(y$dim)))
	    } else
		if (length(y$dim) != length(x$dim))
		    stop("lengths of 'dim' do not conform")
	D <- x$dim
	m <- min(length(D), m)
	if (!identical(y$dim[-m], D[-m]))
	    stop("common parts of 'dim' do not conform")
	if (vector(typeof(x$v), 1L) != vector(typeof(y$v), 1L))
	    stop("definitions of ZERO of 'v' do not conform")
	V <- c(x$v, y$v)
	I <- y$i
	I[, m] <- I[, m] + D[m]
	I <- rbind(x$i, I)
	N <- x$dimnames
	if (!is.null(N[[m]])) {
	    N[[m]] <- 
		c(  N[[m]],
		    if (!is.null(y$dimnames[[m]]))
			y$dimnames[[m]]
		    else
			rep.int("", y$dim[[m]])
		)
	    if (is.null(names(N)))
                names(N) <- names(y$dimnames)
	} else
	    if (!is.null(y$dimnames[[m]])) {
		if (is.null(N))
		    N <- y$dimnames
		else
		    if (is.null(names(N)))
			names(N) <- names(y$dimnames)
		N[[m]] <-
		    c(
			rep.int("", D[m]),
			y$dimnames[[m]]
		     )
	    }
	D[m] <- D[m] + y$dim[m]
	x <- simple_sparse_array(I, V, D, N)
    }
    x
}


###

library("slam")

##
x <- matrix(c(1, 0, 0, 2, 1, 0), nrow = 3)
x
sx <- as.simple_triplet_matrix(x)

y <- matrix(1:6, nrow = 3)
sy <- as.simple_triplet_matrix(y)

identical(tcrossprod(x, y), tcrossprod_simple_triplet_matrix( x, sy))
identical(tcrossprod(x),    tcrossprod_simple_triplet_matrix(sx))
identical(tcrossprod(x, y), tcrossprod_simple_triplet_matrix(sx, sy))
identical(tcrossprod(x, y), tcrossprod_simple_triplet_matrix(sx,  y))

identical(crossprod(x, y),  crossprod_simple_triplet_matrix( x, sy))
identical(crossprod(x),     crossprod_simple_triplet_matrix(sx))
identical(crossprod(x, y),  crossprod_simple_triplet_matrix(sx, sy))
identical(crossprod(x, y),  crossprod_simple_triplet_matrix(sx,  y))

identical(crossprod(x, y),  matprod_simple_triplet_matrix(t( x), sy))
identical(crossprod(x, y),  matprod_simple_triplet_matrix(t(sx), sy))
identical(crossprod(x, y),  matprod_simple_triplet_matrix(t(sx),  y))

## Note that correctness under bailout is covered elsewhere.

##

library("slam")

##
.Call(slam:::R_part_index, factor(rep(1L, 4L)))
.Call(slam:::R_part_index, factor(1:4))
.Call(slam:::R_part_index, factor(c(1L,2L,2L,1L)))
.Call(slam:::R_part_index, factor(c(1L,2L,NA,1L)))

##
i <- 1:27
x <- arrayInd(i, .dim = c(3L,3L,3L))
.Call(slam:::R_vector_index, c(3L,3L,3L), x)
x[14L, 2L] <- NA
.Call(slam:::R_vector_index, c(3L,3L,3L), x)

##
v <- c(1L,1L)
p <- matrix(c(1L,2L,3L, 2L,2L,2L), nrow = 2L, byrow = TRUE)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, v, 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.logical(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.double(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.raw(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.complex(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.character(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.list(v), 2L)
.Call(slam:::R_ini_array, c(3L,3L,3L), p, as.expression(v), 2L)

.Call(slam:::R_ini_array, 3L, c(1L,2L), c(1L,1L), 2L)

.Call(slam:::R_split_col, array(c(1L,2L), c(2L, 2L)))

##
x <- matrix(c(1L,1L,1L,1L,1L,2L,1L,3L,1L,2L), 
	    ncol = 2, byrow = TRUE)
x
.Call(slam:::R_match_matrix, x, NULL, NULL)
.Call(slam:::R_match_matrix, x, x[1:3,], 0L)
.Call(slam:::R_match_matrix, x, matrix(0L, 0, 2), 0L)


##
x <- matrix(c(1L,2L,2L,2L,NA,1L,NA,2L,NA,NA), 
	    ncol = 2, byrow = TRUE)
x
.Call(slam:::R_all_row, x > 1L, FALSE)
.Call(slam:::R_all_row, x > 1L, TRUE)

###

## CB 2013/12
colapply_simple_triplet_matrix <-
function(x, FUN, ...) {
    FUN <- match.fun(FUN)
    out <- .External(R_col_apply_stm, x, FUN, ...)
    if (length(out)) {
	if (all(unlist(lapply(out, length)) == 1L))
	    out <- unlist(out, recursive = FALSE, use.names = FALSE)
	names(out) <- colnames(x)
    } 
    else
	## NOTE we always supplie as matrix in case dimensions 
	##	must conform with further arguments.
	storage.mode(out) <-
	    typeof(FUN(as.matrix(x), ...))
    out
}

rowapply_simple_triplet_matrix <-
function(x, FUN, ...) {
    FUN <- match.fun(FUN)
    if (!is.simple_triplet_matrix(x))
	stop("'x' not of class simple_striplet_matrix")
    colapply_simple_triplet_matrix(t(x), FUN, ...)
}

## FIXME a workaround for a proper C implementation.
crossapply_simple_triplet_matrix <- 
function(x, y = NULL, FUN, ...) {
    FUN <- match.fun(FUN)
    if (is.null(y)) {
	if (!is.simple_triplet_matrix(x))
	    stop("'x' not of class simple_triplet_matrix")
	Y <- x
	out <- colapply_simple_triplet_matrix(x, function(x) {
	out <- colapply_simple_triplet_matrix(Y, FUN, x, ...)
	    Y <<- Y[, -1L]
	    out
	})
	out <- unlist(out, recursive = FALSE, use.names = FALSE)
	Y <- simple_triplet_zero_matrix(x$ncol)
	Y <- row(Y) >= col(Y)
	out[Y] <- out
	out <- matrix(out, nrow = x$ncol, ncol = x$ncol, byrow = TRUE,
	    dimnames = if (!is.null(colnames(x)))
		list(colnames(x), colnames(x))
	)
	out[Y] <- t(out)[Y]
	return(out)
    }
    if (is.simple_triplet_matrix(y)) {
	if (!is.simple_triplet_matrix(x))
	    return(
		t(crossapply_simple_triplet_matrix(y, as.matrix(x), 
		    function(y, x) FUN(x, y, ...)))
	    )
	if (x$nrow != y$nrow)
	    stop("the numer of rows of 'x' and 'y' do not conform")
	## Fix asymmetric performance.
	if (x$ncol > y$ncol)
	    return(
		t(crossapply_simple_triplet_matrix(y, x,
		    function(y, x) FUN(x, y, ...)))
	    )
	if (y$ncol > 0L &&
	    x$ncol > 0L) {
	    out <- colapply_simple_triplet_matrix(x, function(x)
		   colapply_simple_triplet_matrix(y, function(y)
			FUN(x, y, ...)))
	}
	else
	    out <- colapply_simple_triplet_matrix(x[, 0L], 
		FUN, as.matrix(y[, 0L]), ...) 
    }
    else {
	if (!is.simple_triplet_matrix(x))
	    stop("'x, y' not of class simple_triplet_matrix")
	y <- as.matrix(y)
	if (x$nrow != nrow(y))
	    stop("the numer of rows of 'x' and 'y' do not conform")
	if (ncol(y) > 0L &&
	     x$ncol > 0L) {
	    Y <- split(y, factor(col(y), levels = seq_len(ncol(y))))
	    out <- colapply_simple_triplet_matrix(x, function(x) {
		out <- lapply(Y, function(y)
		    FUN(x, y, ...))
		if (all(unlist(lapply(out, length)) == 1L))
		    out <- unlist(out, recursive = FALSE, use.names = FALSE)
		out
	    })
	    rm(Y)
	}
	else
	    out <- colapply_simple_triplet_matrix(x[, 0L], 
		FUN, y[, 0L, drop = FALSE], ...)
    }
    out <- unlist(out, recursive = FALSE, use.names = FALSE)
    out <- matrix(out, nrow = x$ncol, ncol = ncol(y), byrow = TRUE,
	dimnames = 
	    if (!is.null(colnames(x)) || !is.null(colnames(y)))
		list(colnames(x), colnames(y))
    )
    out
}

tcrossapply_simple_triplet_matrix <-
function(x, y = NULL, FUN, ...) {
    FUN <- match.fun(FUN)
    if (is.simple_triplet_matrix(x))
	crossapply_simple_triplet_matrix(t(x),
	    if (is.null(y))
		y
	    else
		if (is.simple_triplet_matrix(y))
		    t(y)
		else
		    t(as.matrix(y)),
	    FUN, ...
    )
    else
	if (is.simple_triplet_matrix(y))
	    crossapply_simple_triplet_matrix(t(as.matrix(x)), t(y), FUN, ...)
	else
	    stop("'x, y' not of class simple_triplet_matrix")
}

###

read_stm_CLUTO <-
function(file)
{
    ## Read CLUTO sparse matrix format.
    
    ## Read in the matrix file.
    l <- strsplit(readLines(file, warn = FALSE), "[[:space:]]+")
    l <- lapply(l, as.double)
    l <- lapply(l, na.omit)

    ## Extract header information.
    nRow <- as.integer(l[[1L]][1L])
    nCol <- as.integer(l[[1L]][2L])
    nElem <- l[[1L]][3L]
    ## Remove header
    l <- l[-1L]

    ## Compute i, j, and v slots for a simple_triplet_matrix.
    rowLen <- lengths(l)
    l <- unlist(l)
    i <- rep.int(seq_len(nRow), rowLen / 2)
    j <- l[seq.int(1, length(l), by = 2)]
    v <- l[seq.int(2, length(l), by = 2)]

    ## Sanity check
    if(nElem != length(v))
        stop("invalid matrix format")

    ## Generate sparse matrix
    m <- simple_triplet_matrix(i, j, v, nRow, nCol)

    if(is.character(file)) {
        ## Use col labels file if available and valid.
        if(file.exists(f <- sprintf("%s.clabel", file))) {
            lines <- readLines(f)
            if(length(lines) == nCol)
                colnames(m) <- lines
        }
        ## Use row labels file if available and valid.
        if(file.exists(f <- sprintf("%s.rlabel", file))) {
            lines <- readLines(f)
            if(length(lines) == nRow)
                rownames(m) <- lines
        }
        ## Use row class file if available.
        if(file.exists(f <- sprintf("%s.rclass", file))) {
            lines <- readLines(f)
            if(length(lines) == nRow)
                attr(m, "rclass") <- lines
        }
    }
        
    m
}

write_stm_CLUTO <-
function(x, file)
{
    ## Write CLUTO sparse matrix format.
    
    x <- as.simple_triplet_matrix(x)

    ## Generate header.
    header <- paste(x$nrow, x$ncol, length(x$v))

    ## Generate content.
    content <- Map(function(u, v) paste(u, v, collapse = " "),
                   split(x$j, x$i),
                   split(x$v, x$i))

    ## Write out.
    writeLines(c(header, unlist(content)), file)

    if(is.character(file)) {
        if(!is.null(rnms <- rownames(x)))
            writeLines(rnms, sprintf("%s.rlabel", file))
        if(!is.null(cnms <- colnames(x)))
            writeLines(cnms, sprintf("%s.clabel", file))
    }
            
}

read_stm_MC <-
function(file, scalingtype = NULL)
{
    ## Read the CCS format variant employed by MC
    ## (http://www.cs.utexas.edu/users/dml/software/mc/) and related
    ## software projects at cs.utexas.edu such as gmeans.
    ## The main MC web page points to
    ## http://www.cs.utexas.edu/users/jfan/dm/README.html 
    ## which no longer seems to exist: but the MC sources contain a file
    ## README with some information.
    ## The basic CCS format is documented in
    ## http://www.cs.utexas.edu/users/inderjit/Resources/sparse_matrices.
    
    d <- scan(sprintf("%s_dim", file), what = integer(0), quiet = TRUE)
    nr <- d[1L]
    nc <- d[2L]

    i <- scan(sprintf("%s_row_ccs", file), what = integer(0),
              quiet = TRUE)
    p <- scan(sprintf("%s_col_ccs", file), what = integer(0),
              quiet = TRUE)
    if(is.null(scalingtype)) {
        ## The name of the file with the non-zero entries varies with
        ## the t-f-n scaling pattern employed (and possibly an 'i' at
        ## the end indicating that row and columne scaling were
        ## performed independently:
        scalingtype <-
            expand.grid(c("t", "l"),
                        c("x", "f", "e", "1"),
                        c("x", "n", "1"),
                        c("", "i"))
        ## (Not sure whether all combinations really make sense.)
        scalingtype <- 
            apply(scalingtype, 1L, paste, collapse = "")
    }
    files <- sprintf("%s_%s_nz", file, scalingtype)
    pos <- which(file.exists(files))[1L]
    x <- scan(files[pos], what = numeric(0), quiet = TRUE)
    scalingtype <- scalingtype[pos]

    ## Sanity check
    if(d[3L] != length(x))
        stop("invalid matrix format")

    ## In special cases (e.g., when CCS was produced by the MC toolkit,
    ## see http://userweb.cs.utexas.edu/users/jfan/dm/README.html) we
    ## can also infer the row and col names.
    rnms <- if(file.exists(f <- sprintf("%s_words", file))) {
        readLines(f)[seq.int(from = 2L, length.out = nr)]
    } else NULL
    cnms <- if(file.exists(f <- sprintf("%s_docs", file))) {
        sub("^[^ ]*: ", "", readLines(f))
    } else NULL

    m <- simple_triplet_matrix(i + 1L,
                               rep.int(seq_len(nc), diff(p)),
                               x,
                               nr, nc, list(rnms, cnms))
    attr(m, "scalingtype") <- scalingtype
    m
}

write_stm_MC <-
function(x, file)
{
    ## Write CCS sparse matrix format as used by MC and other software
    ## projects from cs.utexas.edu such as gmeans.

    ## <FIXME>
    ## This said:

    ## Gmeans uses a compressed column storage (CCS)
    ## See http://www.cs.utexas.edu/users/inderjit/Resources/sparse_matrices
    ##
    ## However since Gmeans clusters along columns, and the input for
    ## our skmeans clusters along rows, we would need to transpose the
    ## matrix first, and then write it to CCS. 
    ##
    ## Instead we could directly write to compressed row storage (CRS)
    ## to avoid the transpose
    ## See
    ## http://netlib.org/linalg/html_templates/node92.html#SECTION00931200000000000000

    ## Does this mean we should not transpose in general, but when
    ## writing out for gmeans in skmeans only?
    ## </FIXME>
    
    x <- t(as.simple_triplet_matrix(x))
    # Based on slam/work/Matrix.R
    ind <- order(x$j, x$i)
    write(paste(nrow(x), ncol(x), length(x$v)),
          sprintf("%s_dim", file))
    write(x$i[ind] - 1L,
          sprintf("%s_row_ccs", file), sep = "\n")
    write(c(0L, cumsum(tabulate(x$j[ind], x$ncol))),
          sprintf("%s_col_ccs", file), sep = "\n")
    write(x$v[ind],
          sprintf("%s_tfn_nz", file), sep = "\n")

    ## Could also try to write a _docs file.
    ## But what does the 2nd half of the _words files contain?
}

## A simple class for sparse arrays.

## Not very useful yet: need at least a subscript method.
## (Unfortunately, additional methods such as for rowSums/colSums or
## apply, etc., are not straightforward to add in an S3 world ...)

simple_sparse_array <-
function(i, v, dim = NULL, dimnames = NULL)
{
    ## See examples
    storage.mode(i) <- "integer"
    if (!is.matrix(i))
	dim(i) <- c(length(i), 1L)
    ## <FIXME>
    ## Add some sanity checking eventually ...
    ## i should be a matrix of indices (non-"zero" entries).
    ## v should be a "vector" of non-zero values, with length equal to
    ## the number of rows of i.
    ## </FIXME>
    if(is.null(dim)) dim <- if(NROW(i)) apply(i, 2L, max) else c(0L, 0L)
    ## <FIXME>
    ## Add checks for dimnames: should be NULL or a list of entries
    ## which are either NULL or character vectors as long as the
    ## corresponding dim.
    ## </FIXME>
    if(anyDuplicated(.Call(R_split_index_matrix_by_rows, i)) > 0)
        stop("Duplicate rows in i are not allowed.")
    ssa <- list(i = i, v = v, dim = as.integer(dim), dimnames = dimnames)
    class(ssa) <- "simple_sparse_array"
    ## Note that this should never be true as it implies that either
    ## the class is wrong or the container is malformed.
    if (!.Call(R__valid_ssa, ssa))
        stop("failed to create a valid 'simple_sparse_array' object")
    ssa
}

as.simple_sparse_array <-
function(x)
    UseMethod("as.simple_sparse_array")

as.simple_sparse_array.simple_sparse_array <- identity

as.simple_sparse_array.array <-
function(x)
{
    x <- unclass(x)
    dx <- dim(x)
    if(!prod(dx))
	return(simple_sparse_array(matrix(integer(), 0L, length(dx)), 
				   c(x), dx, dimnames(x)))
    ind <- which(is.na(x) | (x != vector(typeof(x), 1L)), arr.ind = TRUE)
    dimnames(ind) <- NULL
    simple_sparse_array(ind, x[ind], dx, dimnames(x))
}

as.simple_sparse_array.matrix <- as.simple_sparse_array.array

as.simple_sparse_array.simple_triplet_matrix <-
function(x) 
    simple_sparse_array(cbind(x$i, x$j), x$v, c(x$nrow, x$ncol), dimnames(x))

as.simple_sparse_array.default <-
function(x)
    as.simple_sparse_array(unclass(as.array(x)))

as.array.simple_sparse_array <-
function(x, ...)
{
    v <- x$v
    dim <- x$dim
    y <- array(vector(typeof(v), 1L), dim = dim,
               dimnames = x$dimnames)
    y[x$i] <- v
    y
}

is.simple_sparse_array <-
function(x)
    inherits(x, "simple_sparse_array")

Math.simple_sparse_array <-
function(x, ...)
{
    ## Functions in the Math group mapping 0 to 0:
    funs <- c("abs", "sign", "sqrt",
              "floor", "ceiling", "trunc", "round", "signif")
    if(is.na(match(as.character(.Generic), funs)))
        stop(gettextf("Generic '%s' not defined for \"%s\" objects.",
                      .Generic, .Class),
             domain = NA)

    x$v <- get(.Generic)(x$v, ...)
    x
}

Ops.simple_sparse_array <- 
function(e1, e2) 
{
    stop("Not implemented.")
}

Summary.simple_sparse_array <-
function(..., na.rm = FALSE)
{
    v <- unlist(lapply(list(...),
                       function(e) {
                           v <- as.simple_sparse_array(e)$v
                           if(length(v) < prod(dim(e)))
                               v <- c(v, vector(typeof(v), 1L))
                           v
                       }),
                recursive = FALSE)
    do.call(.Generic, list(v, na.rm = na.rm))
}

dim.simple_sparse_array <-
function(x)
    x$dim

`dim<-.simple_sparse_array` <-
function(x, value)
{
    value <- as.integer(value)
    if(!length(value) || any(is.na(value)))
        stop("invalid dim replacement value")
    dx <- dim(x)
    if(prod(value) != prod(dx))
        stop("invalid dim replacement value")

    x$i <- arrayInd(.Call(R_vector_index, x$dim, x$i), value)
    x$dim <- value
    x$dimnames <- NULL
    
    x
}

dimnames.simple_sparse_array <-
function(x)
    x$dimnames

## FIXME we now have drop_simple_sparse_array
`[.simple_sparse_array` <-
function(x, ...)
{
    ## Note that calling x[] with a simple sparse array x will call the
    ## subscript method with args x and missing ...
    na <- nargs()
    if((na == 1L) || (na == 2L) && missing(..1))
        return(x)

    nd <- length(x$dim)
    pd <- prod(x$dim)

    ## See now matrix.R for comment.
    .protect <-  pd > slam_options("max_dense")

    ## Note there is a limit to representing integer numbers as 
    ## doubles (see above).
    spos <- function(i) {
	if(!nrow(i)) 
	    return(vector(mode = typeof(i), length = 0L))
        ## Scalar positions of array index matrices i in the usual row
        ## major ordering of arrays.
	if(ncol(i) > 1L) {
	    ## This may not work on systems with BLAS issues
	    ## as.vector(tcrossprod(c(1L, cumprod(x$dim[-nd])), i - 1L)) + 1L
	    1L + row_sums((i - 1L) * rep(c(1L, cumprod(x$dim)[-nd]), 
					 each = nrow(i)))
	} else
	    as.vector(i)
    }

    if(na == 2L) {
        i <- ..1
        ## Single index subscripting.
        if(is.logical(i))
            stop("Logical vector subscripting currently not implemented.")
        else if(is.character(i))
            stop("Character vector subscripting currently not implemented.")
        else if(!is.matrix(i)) {
	    if (!is.numeric(unclass(i)))
                stop(gettextf("Invalid subscript type: %s.",
                              typeof(i)),
                     domain = NA)
	    if(log2(pd) > .Machine$double.digits)
	      stop("Numeric vector subscripting disabled for this object.")
	     ## Shortcut
	     if(!length(i)) 
		return(vector(mode = typeof(x$v), length = 0L))
            ## Let's hope we have a vector.
            ## What if we have both negatives and positives?
	    if(is.double(i))
		i <- trunc(i)
            if(all(i >= 0, na.rm = TRUE)) {
                i <- i[i > 0]
                out <- vector(mode = typeof(x$v), length = length(i))
		if(length(out)) {
		    ## Missing values.
		    is.na(i) <- i > pd
		    is.na(out) <- is.na(i)
		    i <- match(i, spos(x$i), 0L)
		    out[i > 0L] <- x$v[i]
		}
            } else if(!any(is.na(i)) && all(i <= 0)) {
		if(.protect)
		  stop("Negative vector subsripting disabled for this object.")
                out <- vector(mode = typeof(x$v), pd)
                out[spos(x$i)] <- x$v
		## NOTE this fails if NAs are introduced by 
		##	coercion to integer. 
                out <- out[i]
            }
            else stop("Cannot mix positive and negative subscripts.")
        }
        else {
	     ## Shortcut
	     if(!nrow(i)) 
		return(vector(mode = typeof(x$v), length = 0L))
	     ## Ignore dimensions. 
	     if(ncol(i) != nd || !is.numeric(i)) 
		return(do.call(`[.simple_sparse_array`, 
			       list(x = x, as.vector(i))))
	    if(is.double(i))
		i <- trunc(i)
	    k <- .Call(R_all_row, i > 0, FALSE)
            i <- i[k, , drop = FALSE]
            ## Note that negative values are not allowed in a matrix
            ## subscript.
            if(any(i < 0, na.rm = TRUE))
                stop("Invalid subscript.")
            out <- vector(mode = typeof(x$v), length = nrow(i))
	    if(length(out)) {
		if(any(i > rep(x$dim, each = nrow(i)), na.rm = TRUE))
		    stop("subscript out of bounds")
		## Missing values.
		k <- k[k]
		is.na(out) <- is.na(k)
		rm(k)
		## This is not really the fastest way to match rows, but is
		## there an obvious better one?
		## pos <- match(split(i, row(i)), split(x$i, row(x$i)), 0L)
		storage.mode(i) <- "integer"
		i <- .Call(R_match_matrix, x$i, i, 0L)[[2L]]
		out[i > 0L] <- x$v[i]
	    }
        }
    }
    else {
        if(na != (nd + 1L))
            stop("Incorrect number of dimensions.")
        ## Get indices.
	args <- vector("list", na - 1L)
	for(k in seq_along(args)) {
	    n <- as.name(sprintf("..%i", k))
	    if (!do.call(missing, list(n)))
		args[[k]] <- eval(n)
	}
        ## Ready to go.
        dx <- x$dim
        pos <- rep.int(TRUE, length(x$v))
        ind <- vector("list", length = nd)
        for(k in seq_len(nd)) {
            i <- args[[k]]              # Given indices.
            if(is.null(i)) {
		ind[[k]] <- seq_len(dx[k])
		next
	    }
            else if(!is.numeric(i))
                stop("Only numeric multi-index subscripting is implemented.")
            else {
		if (any(is.na(i)))
		    stop("NA indices currently not allowed")
		if(is.double(i))
		    i <- trunc(i)
                if(all(i >= 0)) {
                    i <- i[i > 0]
                    if(any(duplicated(i)))
                        stop("Repeated indices currently not allowed.")
		    if(any(i > dx[k]))
			stop("subscript out of bounds")
                } else if(all(i <= 0))
		    ## NOTE this fails if NAs are introduced by 
		    ##	    coercion to integer. 
                    i <- seq_len(dx[k])[i]
                else
                    stop("Cannot mix positive and negative subscripts.")
                ind[[k]] <- i
                dx[k] <- length(i)
                j <- match(x$i[, k], i, 0L)
                x$i[j > 0L, k] <- seq_along(i)[j]
                pos <- pos & (j > 0L)
            }
        }
        if(!is.null(dnx <- x$dimnames))
            dnx[] <- Map(`[`, dnx, ind)
        out <- simple_sparse_array(x$i[pos, , drop = FALSE], x$v[pos],
                                   dx, dnx)
    }

    out

}

## <TODO>
## Add duplicated and unique methods for simple sparse arrays along the
## lines of the corresponding methods for simple triplet matrices.
## </TODO>

print.simple_sparse_array <-
function(x, ...)
{
    writeLines(gettextf("A simple sparse array of dimension %s.",
                        paste(dim(x), collapse = "x")))
    invisible(x)
}

mean.simple_sparse_array <-
function(x, ...)
{
    sum(x$v) / prod(dim(x))
}

aperm.simple_sparse_array <-
function(a, perm = NULL, ...)
{
    s <- seq_along(a$dim)
    if(is.null(perm))
        perm <- rev(s)
    else {
        perm <- if(is.character(perm))
            match(perm, names(a$dimnames))
        else if(is.numeric(perm))
            match(perm, s)
        else NULL
        if(length(perm) != length(s) || any(is.na(perm)))
            stop("Invalid permutation.")
    }
    simple_sparse_array(a$i[, perm, drop = FALSE], a$v,
                        a$dim[perm], a$dimnames[perm])
    
}

as.vector.simple_sparse_array <-
function(x, mode = "any")
    as.vector(as.array(x), mode)

simple_sparse_zero_array <-
function(dim, mode = "double")
{
    ld <- length(dim)
    if (!ld)
	stop("'dim' must have positive length")
    simple_sparse_array(matrix(integer(), 0L, ld), vector(mode, 0L), dim)
}

###

## For performance reasons the constructor does not
## check for multiple or 'zero' elements.
##
## Argument 'strict' provides a choice whether to 
## enforce these constraints, or to reduce 'multiples'
## to NA (unless they all are identical) and remove
## 'zeros'.
##
reduce_simple_sparse_array <- 
function(x, strict = FALSE, order = FALSE) 
{
    if (!.Call(R__valid_ssa, x))
	stop("'x' not of class 'simple_sparse_array'")
    I <- x$i
    if (length(i <- attributes(I)) > 1L)
	dim(I) <- i$dim
    rm(i)
    V <- .Call(R_unattr, x$v)
    if (length(V)) {
	## reduce multiple entries
	i <- .Call(R_match_matrix, I, NULL, NULL)
	if (length(i[[1L]]) > length(i[[2L]])) {
	    if (strict)
		stop("multiple entries")
	    I <- I[i[[2L]],, drop = FALSE]
	    i <- i[[1L]]
	    attributes(i) <-
		list(levels = seq_len(dim(I)[1L]), 
		     class  = "factor")
	    V <- split(V, i)
	    rm(i)
	    names(V) <- NULL
            nas <- FALSE
	    V <- sapply(V, function(x) 
		if (length(x) > 1L) {
                    x <- unique(x)
                    if(length(x) > 1L) {
                        t <- typeof(x)
                        if(t == "raw")
                            stop("cannot reduce multiple entries (missing not defined")
                        else {
                            nas <<- TRUE
                            as.vector(NA, t)
                        }
                    } else
                        x
                } else 
                    x,
              
		USE.NAMES = FALSE)
            if(nas)
                warning("NAs introduced by reduction")
	} else
	    rm(i)
	## remove 'zero' entries
	i <- which(V == vector(typeof(V), 1L))
	if (strict)
	    stop("zero entries")
	if (length(i)) {
	    i <- -i
	    V <- V[i]
	    I <- I[i,, drop = FALSE]
	}
	rm(i)
	## order entries
	if (order) {
	    i <- do.call(base::order, rev(.Call(R_split_col, I)))
	    if (!identical(i, seq_along(i))) {
		V <- V[i]
		I <- I[i,, drop = FALSE]
	    }
	    rm(i)
	}
    }
    D <- as.vector(x$dim)
    N <- x$dimnames
    N <-
    if (!length(N) ||
	(is.null(names(N)) &&
	  all(vapply(N, is.null, NA))))
	NULL
    else
	lapply(N, as.vector)
    simple_sparse_array(I, V, D, N)
}

##
drop_simple_sparse_array <-
function(x) 
{
    if (!is.simple_sparse_array(x))
	stop("'x' not of class 'simple_sparse_array'")
    dx <- x$dim
    if (any(dx == 0L))
	return(vector(typeof(x$v), 0L))	    ## sanitize
    k <- which(dx == 1L)
    if (length(k) == length(dx))
	return(x$v)
    if (length(k)) {
	k <- -k
	x$i <- x$i[, k, drop = FALSE]
	x$dim <- dx[k]
	if (!is.null(x$dimnames))
	    x$dimnames <- x$dimnames[k]
    }
    x
}


## see simplify2array
simplify_simple_sparse_array <-
function(x, higher = TRUE)
{
    if (!is.simple_sparse_array(x))
	stop("'x' not of class 'simple_sparse_array'")
    V <- x$v
    if (is.atomic(V) || 
	  !length(V))
	return(x)
    i <- unique(unlist(lapply(V, length)))
    ## FIXME not implemented
    if (length(i) > 1L) 
	return(x)
    if (!i)
	return(x)
    if (i == 1L) {
	x$v <- unlist(V, recursive = FALSE)
	return(x)
    }
    I <- x$i
    D <- x$dim
    N <- x$dimnames
    if (higher &&
	length(d <- unique(lapply(V, dim))) == 1L &&
      !is.null(d <- unlist(d))) {
	i <- d
	n <- dimnames(V[[1L]])
    } else 
	if (!is.null(n <- names(V[[1L]])))
	    n <- list(n)
    V <- unlist(V, recursive = FALSE)
    ## FIXME not optimized
    for (k in rev(i)) {
	l <- dim(I)[1L]
	if (k > 1L)
	    I <- apply(I, 2L, rep, each = k)
	I <- cbind(rep.int(seq.int(k), l), I)
    }
    if (!is.null(N)) {
	if (!is.list(n))
	    n <- list(n)
	N <- c(n, N)
    } else
	if (!is.null(n)) 
	    N <- list(n, vector("list", length(D)))
    D <- c(i, D)
    simple_sparse_array(I, V, D, N)
}


###

## NOTE the C code must always check for special values and
##      therefore has control over how to proceed. For now
##      it calls the bailout function below. 
##
##	For verbose information set the verbose argument to 
##	TRUE. Transposition of the return value (!) is only
##	implemented for dense.
##
##      The general case is now also handled in C. Runtime
##      could be further improved if the data need not be 
##      ordered (see the C code).

.tcrossprod_simple_triplet_matrix <-
function(x, y = NULL, transpose = FALSE, bailout = TRUE, verbose = FALSE) {
    if (!is.simple_triplet_matrix(x))
	stop("'x' not of class simple_triplet_matrix")
    if (is.null(y) ||
	is.simple_triplet_matrix(y)) {
	if (transpose)
	    stop("'transpose' not implemented")
	.Call(R_tcrossprod_stm_stm, x, y,
	    if (bailout)
		environment(.tcrossprod_simple_triplet_matrix),
	    verbose
	)
    }
    else
	.Call(R_tcrossprod_stm_matrix, x,
	    as.matrix(y),
	    if (bailout)
		environment(.tcrossprod_simple_triplet_matrix),
	    verbose,
	    transpose
	)

}

.tcrossprod_bailout <-
function(x, y, transpose) {
    if (transpose)
	## see above
	base::tcrossprod(y, as.matrix(x))
    else
	base::tcrossprod(as.matrix(x),
	    if (is.null(y))
		y
	    else
		as.matrix(y)
	)
}

## Used by package skmeans.
.ttcrossprod_simple_triplet_matrix <-
function(x, y = NULL)
    .tcrossprod_simple_triplet_matrix(x, y, TRUE)

##
tcrossprod_simple_triplet_matrix <-
function(x, y = NULL) {
    if(is.simple_triplet_matrix(x)) {
        if(!is.simple_triplet_matrix(y) &&
           .is_sparse_mat_coercible_to_stm(y))
            y <- as.simple_triplet_matrix(y)
        .tcrossprod_simple_triplet_matrix(x, y)
    }
    else if(is.simple_triplet_matrix(y)) {
        x <- if(.is_sparse_mat_coercible_to_stm(x))
                 as.simple_triplet_matrix(x)
             else
                 as.matrix(x)
        .tcrossprod_simple_triplet_matrix(y, x, TRUE)
    }
    else
        stop("neither 'x' nor 'y' of class 'simple_triplet_matrix'")
}

crossprod_simple_triplet_matrix <-
function(x, y = NULL) {
    if(is.simple_triplet_matrix(x)) {
        y <- if(is.null(y))
                 y
             else if(is.simple_triplet_matrix(y))
                 t(y)
             else if(.is_sparse_mat_coercible_to_stm(y))
                 t(as.simple_triplet_matrix(y))
             else
                 t(as.matrix(y))
        .tcrossprod_simple_triplet_matrix(t(x), y)
    }
    else if(is.simple_triplet_matrix(y)) {
        x <- if(.is_sparse_mat_coercible_to_stm(x))
                 as.simple_triplet_matrix(x)
             else
                 as.matrix(x)
        .tcrossprod_simple_triplet_matrix(t(y), t(x), TRUE)
    }
    else
        stop("neither 'x' nor 'y' of class 'simple_triplet_matrix'")
}

matprod_simple_triplet_matrix <-
function(x, y) {
    if(is.simple_triplet_matrix(x)) {
        y <- if(is.simple_triplet_matrix(y))
                 y
             else if(.is_sparse_mat_coercible_to_stm(y))
                 as.simple_triplet_matrix(y)
             else
                 as.matrix(y)
	.tcrossprod_simple_triplet_matrix(x, t(y))
    }
    else if(is.simple_triplet_matrix(y)) {
        x <- if(.is_sparse_mat_coercible_to_stm(x))
                 as.simple_triplet_matrix(x)
             else
                 as.matrix(x)
        .tcrossprod_simple_triplet_matrix(t(y), x, TRUE)
    }
    else
        stop("neither 'x' nor 'y' of class 'simple_triplet_matrix'")
}

##

matrixOps.simple_triplet_matrix <-
function(x, y)
{
    switch(.Generic,
           "%*%" = matprod_simple_triplet_matrix(x, y),
           "crossprod" = if(missing(y))
                             crossprod_simple_triplet_matrix(x)
                         else
                             crossprod_simple_triplet_matrix(x, y),
           "tcrossprod" = if(missing(y))
                              tcrossprod_simple_triplet_matrix(x)
                          else
                              tcrossprod_simple_triplet_matrix(x, y))
}

chooseOpsMethod.simple_triplet_matrix <- 
function(x, y, mx, my, cl, reverse)
    TRUE

\name{rollup}
% NOTE need generics
\alias{rollup}
\alias{rollup.array}
\alias{rollup.matrix}
\alias{rollup.simple_triplet_matrix}
\alias{rollup.simple_sparse_array}
\title{
Rollup Sparse Arrays
}
\description{
Rollup (aggregate) sparse arrays along arbitrary dimensions.
}
\usage{
rollup(x, MARGIN, INDEX, FUN, ...)

\method{rollup}{simple_triplet_matrix}(x, MARGIN, INDEX = NULL, FUN = sum, \dots,
       REDUCE = FALSE)
\method{rollup}{simple_sparse_array}(x, MARGIN, INDEX = NULL, FUN = sum, \dots, 
       DROP = FALSE, EXPAND = c("none", "sparse", "dense", "all"), 
       MODE = "double")
\method{rollup}{matrix}(x, MARGIN, INDEX = NULL, FUN = sum, \dots,
       DROP = FALSE, MODE = "double")
\method{rollup}{array}(x, MARGIN, INDEX = NULL, FUN = sum, \dots, 
       DROP = FALSE, MODE = "double")
}
\arguments{
  \item{x}{a sparse (or dense) array, typically of numeric or logical values.
}
  \item{MARGIN}{a vector giving the subscripts (names) of the dimensions to be rolled up.
}
  \item{INDEX}{a corresponding (\code{list} of) \code{factor} (components) in the sense that \code{as.factor} defines the grouping(s). If \code{NULL} collapses the corresponding dimension(s) (default).}
  \item{FUN}{the name of the function to be applied.
}
  \item{REDUCE}{option to remove zeros from the result.}
  \item{DROP}{option to delete the dimensions of the result which have only one level.}
  \item{EXPAND}{the cell expansion method to use (see Details).
}
  \item{MODE}{the type to use for the values if the result is empty.
}
  \item{\dots}{optional arguments to \code{FUN}.
}
}
\details{
Provides aggregation of sparse and dense arrays, in particular fast summation over the rows or columns of sparse matrices in
\code{simple_triplet}-form.

If (a component of) \code{INDEX} contains \code{NA} values the
corresponding parts of \code{x} are omitted.

For \code{simple_sparse_array} the following cell expansion methods are
provided:
\describe{
    \item{\code{none}:}{The \emph{non-zero} entries of a cell, if any, are
	supplied to \code{FUN} as a \code{vector}.}
    \item{\code{sparse}:}{The number of \emph{zero} entries of a cell is 
	supplied in addition to above, as a second argument.}
    \item{\code{dense}:}{Cells with \emph{non-zero} entries are expanded to
	a dense \code{array} and supplied to \code{FUN}.}
    \item{\code{all}:}{All cells are expanded to a dense \code{array}
	and supplied to \code{FUN}.}
}
Note that the memory and time consumption increases with the level
of expansion.

Note that the default method tries to coerce \code{x} to \code{array}.
}
\value{
An object of the same class as \code{x} where for class
\code{simple_triplet_matrix} the values are always of type \code{double}
if \code{FUN = sum} (default).

The \code{dimnames} corresponding to \code{MARGIN} are based on (the 
components of) \code{INDEX}.
}
\author{
Christian Buchta
}
\note{
Currently most of the code is written in R and, therefore, the memory
and time it consumes is not optimal.
}
\seealso{
\code{\link{simple_triplet_matrix}} and \code{\link{simple_sparse_array}}
for sparse arrays.

\code{\link{apply}} for dense arrays.
}
\examples{
##
x <- matrix(c(1, 0, 0, 2, 1, NA), nrow = 2, 
	    dimnames = list(A = 1:2, B = 1:3))
x
apply(x, 1L, sum, na.rm = TRUE)
##
rollup(x, 2L, na.rm = TRUE)
##
rollup(x, 2L, c(1,2,1), na.rm = TRUE)
## omit
rollup(x, 2L, c(1,NA,1), na.rm = TRUE)
## expand
a <- as.simple_sparse_array(x)
a
r <- rollup(a, 1L, FUN = mean, na.rm = TRUE, EXPAND = "dense")
as.array(r)
## 
r <- rollup(a, 1L, FUN = function(x, nz) 
	length(x) / (length(x) + nz), 
    EXPAND = "sparse"
)
as.array(r)
}
\keyword{array}
\keyword{algebra}
\keyword{arith}

\name{sums}
% NOTE need generics
\alias{row_sums}
\alias{row_sums.simple_triplet_matrix}
\alias{col_sums}
\alias{col_sums.simple_triplet_matrix}
\alias{row_means}
\alias{row_means.simple_triplet_matrix}
\alias{col_means}
\alias{col_means.simple_triplet_matrix}
\title{
Form Row and Column Sums and Means
}
\description{
Form row and column sums and means for sparse arrays (currently
\code{simple_triplet_matrix} only).
}
\usage{
row_sums(x, na.rm = FALSE, dims = 1, \dots)
col_sums(x, na.rm = FALSE, dims = 1, \dots)
row_means(x, na.rm = FALSE, dims = 1, \dots)
col_means(x, na.rm = FALSE, dims = 1, \dots)

\method{row_sums}{simple_triplet_matrix}(x, na.rm = FALSE, dims = 1, \dots)
\method{col_sums}{simple_triplet_matrix}(x, na.rm = FALSE, dims = 1, \dots)
\method{row_means}{simple_triplet_matrix}(x, na.rm = FALSE, dims = 1, \dots)
\method{col_means}{simple_triplet_matrix}(x, na.rm = FALSE, dims = 1, \dots)
}
\arguments{
  \item{x}{a sparse array containing numeric, integer, or logical values.
}
  \item{na.rm}{logical. Should missing values (including \code{NaN})
    be omitted from the calculations?
}
  \item{dims}{currently not used for sparse arrays.
}
  \item{\dots}{currently not used for sparse arrays.
}
}
\details{
Provides fast summation over the rows or columns of sparse matrices in
\code{simple_triplet}-form.
}
\value{
A numeric (double) array of suitable size, or a vector if the result
is one-dimensional. The \code{dimnames} (or \code{names} for a vector
result) are taken from the original array.
}
\author{
Christian Buchta
}
\note{
Results are always of storage type \code{double} to avoid (integer)
overflows.
}
\seealso{
\code{simple_triplet_matrix}, \code{\link[base]{colSums}} for dense
numeric arrays.
}
\examples{
##
x <- matrix(c(1, 0, 0, 2, 1, NA), nrow = 3)
x
s <- as.simple_triplet_matrix(x)
row_sums(s)
row_sums(s, na.rm = TRUE)
col_sums(s)
col_sums(s, na.rm = TRUE)
}
\keyword{array}
\keyword{algebra}
\keyword{arith}

\name{simple_sparse_array}
\alias{simple_sparse_array}
\alias{simple_sparse_zero_array}
\alias{as.simple_sparse_array}
\alias{is.simple_sparse_array}
\alias{simple sparse array}

\alias{simplify_simple_sparse_array}
\alias{reduce_simple_sparse_array}
\alias{drop_simple_sparse_array}

\title{Simple Sparse Arrays}
\description{Data structures and operators for sparse arrays based on
  a representation by index matrix and value vector.}

\usage{
simple_sparse_array(i, v, dim = NULL, dimnames = NULL)
simple_sparse_zero_array(dim, mode = "double")

as.simple_sparse_array(x)
is.simple_sparse_array(x)

simplify_simple_sparse_array(x, higher = TRUE)
reduce_simple_sparse_array(x, strict = FALSE, order = FALSE)
drop_simple_sparse_array(x)
}
\arguments{
\item{i}{Integer matrix of array indices.}
\item{v}{Vector of values.}
\item{dim}{Integer vector specifying the size of the dimensions.}
\item{dimnames}{either \code{NULL} or the names for the dimensions.
          This is a list
          with one component for each dimension, either \code{NULL} or a
          character vector of the length given by \code{dim} for that
          dimension.  The list can be named, and the list names will be
          used as names for the dimensions.  If the list is shorter
          than the number of dimensions, it is extended by \code{NULL}'s to
          the length required.}
\item{mode}{Character string specifying the mode of the values.}
\item{x}{An \R object; an object of class \code{simple_sparse_array}
	  (see Note).}
\item{higher}{Option to use the dimensions of the values (see Note).}
\item{strict}{Option to treat violations of sparse representation as error
	  (see Note).}
\item{order}{Option to reorder elements (see Note).}
}
\details{
  \code{simple_sparse_array} is a generator for
  a class of \dQuote{lightweight} sparse arrays, 
  represented by index matrices and value vectors.
  Currently, only methods for indexing and coercion are implemented.

  \code{simple_sparse_zero_array} is a convenience function for the
  creation of empty arrays.
}
\note{
  The \emph{zero} element is defined as \code{vector(typeof(v), 1L)},
  for example, \code{FALSE} for \code{logical} values (see
  \code{\link{vector}}). Clearly, sparse arrays should not contain
  \emph{zero} elements, however, for performance reasons the class
  generator does not remove them.

  If \code{strict = FALSE} (default) \code{reduce_simple_sparse_array} tries
  to repair violations of sparse representation (\emph{zero, multiple}
  elements), otherwise it stops. If \code{order = TRUE} the elements are
  further reordered (see \code{\link{array}}).

  \code{simplify_simple_sparse_array} tries to reduce \code{v}. If
  \code{higher = TRUE} (default) augments \code{x} by the common dimensions
  of \code{v} (from the left), or the common length. Note that \emph{scalar}
  elements are never extended and unused dimensions never dropped.

  \code{drop_simple_sparse_array} drops unused dimensions.

  If \code{prod(dim(x)) > slam_options("max_dense")} empty and negative
  indexing are disabled
  for \code{[} and \code{[<-}. Further, non-negative single (vector)
  indexing is limited to 52 bits of representation.
}
\seealso{
  \code{\link{simple_triplet_matrix}} for sparse matrices.

  \code{\link{slam_options}} for options.
}
\examples{
x <- array(c(1, 0, 0, 2, 0, 0, 0, 3), dim = c(2, 2, 2))
s <- as.simple_sparse_array(x)
identical(x, as.array(s))

simple_sparse_array(matrix(c(1, 3, 1, 3, 1, 3), nrow = 2), c(1, 2))
}
\keyword{math}

\name{simple_triplet_matrix}
\alias{simple_triplet_matrix}
\alias{simple_triplet_zero_matrix}
\alias{simple_triplet_diag_matrix}
\alias{as.simple_triplet_matrix}
\alias{is.simple_triplet_matrix}
\alias{simple triplet matrix}

\title{Simple Triplet Matrix}
\description{Data structures and operators for sparse matrices based on
  simple triplet representation.}

\usage{
simple_triplet_matrix(i, j, v, nrow = max(i), ncol = max(j),
                      dimnames = NULL)
simple_triplet_zero_matrix(nrow, ncol = nrow, mode = "double")
simple_triplet_diag_matrix(v, nrow = length(v))

as.simple_triplet_matrix(x)
is.simple_triplet_matrix(x)
}
\arguments{
\item{i, j}{Integer vectors of row and column indices, respectively.}
\item{v}{Vector of values.}
\item{nrow, ncol}{Integer values specifying the number of rows and
columns, respectively. Defaults are the maximum row and column indices, respectively.}
\item{dimnames}{A \code{dimnames} attribute for the matrix:
                \code{NULL} or a \code{list} of
                length 2 giving the row and column names respectively.  An
                empty list is treated as \code{NULL},
                and a list of length one as
                row names.  The list can be named, and the list names will be
                used as names for the dimensions.}
\item{mode}{Character string specifying the mode of the values.}
\item{x}{An \R object.}
}
\details{
  \code{simple_triplet_matrix} is a generator for
  a class of \dQuote{lightweight} sparse matrices, \dQuote{simply}
  represented by triplets \code{(i, j, v)} of row indices \code{i}, column
  indices \code{j}, and values \code{v}, respectively.
  \code{simple_triplet_zero_matrix} and
  \code{simple_triplet_diag_matrix} are convenience functions for the
  creation of empty and diagonal matrices.

  Currently implemented operations include the addition, subtraction,
  multiplication and division of compatible simple triplet matrices,
  as well as the multiplication and division of a simple triplet matrix
  and a vector.  Comparisons of the elements of a simple triplet
  matrices with a number are also provided. In addition, 
  methods for indexing, combining by rows
  (\code{rbind}) and columns (\code{cbind}), transposing (\code{t}),
  concatenating (\code{c}), and detecting/extracting duplicated and
  unique rows are implemented.
}
\seealso{
  \code{\link{simple_sparse_array}} for sparse arrays.
}
\examples{
x <- matrix(c(1, 0, 0, 2), nrow = 2)
s <- as.simple_triplet_matrix(x)
identical(x, as.matrix(s))

simple_triplet_matrix(c(1, 4), c(1, 2), c(1, 2))
simple_triplet_zero_matrix(3)
simple_triplet_diag_matrix(1:3)

cbind(rbind(s, t(s)), rbind(s, s))
\dontrun{
## map to default Matrix class
stopifnot(require("Matrix"))
sparseMatrix(i = s$i, j = s$j, x = s$v, dims = dim(s), 
	     dimnames = dimnames(s))
}
}
\keyword{math}

\name{abind}
\alias{abind_simple_sparse_array}
\alias{extend_simple_sparse_array}

\title{Combine Sparse Arrays}
\description{Combine a sequence of (sparse) arrays, matrices, or vectors
  into a single sparse array of the same or higher dimension.}

\usage{

abind_simple_sparse_array(..., MARGIN = 1L)
extend_simple_sparse_array(x, MARGIN = 0L)
}
\arguments{
\item{\dots}{\R objects of (or coercible to) class \code{simple_sparse_array}.}
\item{MARGIN}{The dimension along which to bind the arrays.}
\item{x}{An object of class \code{simple_sparse_array}.}
}
\details{
  \code{abind_simple_sparse_array} automatically extends the dimensions
  of the elements of \sQuote{\dots} before it combines them along the
  dimension specified in \code{MARGIN}. If a negative value is specified
  first all elements are extended left of the target dimension.
  
  \code{extend_simple_sparse_array} inserts one (or more) one-level
  dimension(s) into \code{x} to the right of the position(s) specified in
  \code{MARGIN}, or to the left if specified in negative terms. Note that
  the target positions must all be in the range of the dimensions of \code{x}
  (see Examples).
}
\value{
  An object of class \code{simple_sparse_array} where the \code{dimnames}
  are taken from the elements of \sQuote{\dots}.
}
\author{
Christian Buchta
}
\seealso{
  \code{\link{simple_sparse_array}} for sparse arrays.
}
\examples{
## automatic
abind_simple_sparse_array(1:3, array(4:6, c(1,3)))
abind_simple_sparse_array(1:3, array(4:6, c(3,1)), MARGIN = 2L)

## manual
abind_simple_sparse_array(1:3, 4:6)
abind_simple_sparse_array(1:3, 4:6, MARGIN = -2L)   ## by columns
abind_simple_sparse_array(1:3, 4:6, MARGIN = -1L)   ## by rows

##
a <- as.simple_sparse_array(1:3)
a
extend_simple_sparse_array(a, c( 0L, 1L))
extend_simple_sparse_array(a, c(-1L,-2L))   ## the same
extend_simple_sparse_array(a, c( 1L, 1L))
}
\keyword{array}

\name{crossprod}
\alias{tcrossprod_simple_triplet_matrix}
\alias{crossprod_simple_triplet_matrix}
\alias{matprod_simple_triplet_matrix}
\title{
Matrix Crossproduct
}
\description{
Compute the matrix cross-product of a sparse and a dense or sparse matrix.
}
\usage{
tcrossprod_simple_triplet_matrix(x, y = NULL)
##
crossprod_simple_triplet_matrix(x, y = NULL)
matprod_simple_triplet_matrix(x, y)
}
\arguments{
  \item{x, y}{a matrix in \code{simple_triplet_matrix}-form and/or a dense
matrix, where one must be of the form indicated by the suffix.
}
}
\details{
Function \code{tcrossprod_simple_triplet_matrix} implements fast computation 
of \code{x \%*\% t(x)} and \code{x \%*\% t(y)} (\code{tcrossprod}). The
remaining functions are (optimized) wrappers.
}
\value{
A double matrix, with appropriate \code{dimnames} taken from \code{x}
and \code{y}.
}
\author{
Christian Buchta
}
\note{
The computation is delegated to \code{tcrossprod} if \code{y} (or \code{x}
if \code{y == NULL}) contains any of the special values \code{NA},
\code{NaN}, or \code{Inf}.
}
\seealso{
\code{\link[base]{crossprod}} for dense-on-dense computations.
}
\examples{
## 
x <- matrix(c(1, 0, 0, 2, 1, 0), nrow = 3)
x
s <- as.simple_triplet_matrix(x)
tcrossprod_simple_triplet_matrix(s, x)
##
tcrossprod_simple_triplet_matrix(s)
##
tcrossprod_simple_triplet_matrix(s[1L, ], s[2:3, ])
}
\keyword{algebra}
\keyword{array}

\name{foreign}
\alias{read_stm_CLUTO}
\alias{write_stm_CLUTO}
\alias{read_stm_MC}
\alias{write_stm_MC}
\title{Read and Write Sparse Matrix Format Files}
\description{
  Read and write CLUTO sparse matrix format files, or the CCS format
  variant employed by the MC toolkit.
}
\usage{
read_stm_CLUTO(file)
write_stm_CLUTO(x, file)
read_stm_MC(file, scalingtype = NULL)
write_stm_MC(x, file)
}
\arguments{
  \item{file}{a character string with the name of the file to read or
    write.}
  \item{x}{a matrix object.}
  \item{scalingtype}{a character string specifying the type of scaling
    to be used, or \code{NULL} (default), in which case the scaling will
    be inferred from the names of the files with non-zero entries found
    (see \bold{Details}).
  }
}
\details{
  Documentation for CLUTO including its sparse matrix format used to be 
  available from \samp{https://www-users.cse.umn.edu/~karypis/cluto/}.

  \code{read_stm_CLUTO} reads CLUTO sparse matrices, returning a
  \link{simple triplet matrix}.

  \code{write_stm_CLUTO} writes CLUTO sparse matrices.
  Argument \code{x} must be coercible to a simple triplet matrix via
  \code{\link{as.simple_triplet_matrix}}.

  MC is a toolkit for creating vector models from text documents (see 
  \url{https://www.cs.utexas.edu/~dml/software/mc/}).  It employs a
  variant of Compressed Column Storage (CCS) sparse matrix format,
  writing data into several files with suitable names: e.g., a file with
  \file{_dim} appended to the base file name stores the matrix
  dimensions.  The non-zero entries are stored in a file the name of
  which indicates the scaling type used: e.g., \file{_tfx_nz} indicates
  scaling by term frequency (\samp{t}), inverse document frequency
  (\samp{f}) and no normalization (\samp{x}).  See \file{README} in the
  MC sources for more information.

  \code{read_stm_MC} reads such sparse matrix information with argument
  \code{file} giving the path with the base file name, and returns a
  \link[slam]{simple triplet matrix}.

  \code{write_stm_MC} writes matrices in MC CCS sparse matrix format.
  Argument \code{x} must be coercible to a simple triplet matrix via
  \code{\link[slam]{as.simple_triplet_matrix}}.
}
\keyword{IO}

\name{norms}
\alias{row_norms}
\alias{col_norms}
\title{Row and Column Norms}
\description{
  Compute row and column \eqn{p}-norms.
}
\usage{
row_norms(x, p = 2)
col_norms(x, p = 2)
}
\arguments{
  \item{x}{a sparse \code{\link{simple_triplet_matrix}}, or a dense
    matrix.}
  \item{p}{a numeric at least one.  Using \code{Inf} gives the maximum
    norm.}
}
\value{
  A vector with the row or column \eqn{p}-norms for the given matrix.
}
\examples{
x <- matrix(1 : 9, 3L)
## Row lengths:
row_norms(x)
## Column maxima:
col_norms(x, Inf)
}

\name{options}
\alias{slam_options}
\title{Options for the `slam' package}
\description{Function for getting and setting options for the
  \pkg{slam} package.}
\usage{
slam_options(option, value)
}
\arguments{
  \item{option}{character string indicating the option to get
    or set (see details). If missing, all options are returned as a list.}
  \item{value}{Value to be set. If omitted, the current value is returned.}
}
\details{
  Currently, the following options are available:

  \describe{
    \item{\code{"max_dense"}:}{numeric specifying the maximum \code{length}
      of dense vectors (default: \code{2^24}).
    }
  }
}
\seealso{
  \code{\link{simple_sparse_array}}
}
\examples{
## save defaults
.slam_options <- slam_options()
.slam_options

slam_options("max_dense", 2^25)
slam_options("max_dense")

## reset
slam_options("max_dense", .slam_options$max_dense)
}
\keyword{math}

\name{apply}
\alias{rowapply_simple_triplet_matrix}
\alias{colapply_simple_triplet_matrix}
\alias{crossapply_simple_triplet_matrix}
\alias{tcrossapply_simple_triplet_matrix}
\title{
Apply Functions Over Sparse Matrix Margins
}
\description{
Apply functions to (the cross-pairs of) the rows or columns of a sparse
matrix.
}
\usage{
rowapply_simple_triplet_matrix(x, FUN, ...)
colapply_simple_triplet_matrix(x, FUN, ...)

crossapply_simple_triplet_matrix(x, y = NULL, FUN, ...) 
tcrossapply_simple_triplet_matrix(x, y = NULL, FUN, ...)
}
\arguments{
\item{x, y}{a matrix in \code{simple_triplet_matrix}-form or, one of
\code{x} and \code{y}, of class \code{matrix}.
}
  \item{FUN}{the name of the function to be applied.
}
  \item{\dots}{optional arguments to \code{FUN}.
}
}
\details{
\code{colapply_simple_triplet_matrix} temporarily expands each column of
\code{x} to dense \code{vector} representation and applies the function
specified in \code{FUN}.

\code{crossapply_simple_triplet_matrix} temporarily expands each cross-pair
of columns of \code{x} (and \code{y}) to dense \code{vector} representation
and applies the function specified in \code{FUN}.

Note that if \code{y = NULL} then only the entries in the lower triangle
and the diagonal are computed, assuming that \code{FUN} is symmetric.
}
\value{
A \code{vector} (\code{matrix}) of length (dimensionality) of the margin(s)
used. The type depends on the result of \code{FUN}.

Note that the result of \code{colapply_simple_triplet_matrix} is never
simplified to \code{matrix}.
}
\author{
Christian Buchta
}
%\note{
%}
\seealso{
\code{\link[base]{apply}} for dense-on-dense computations.
}
\examples{
## 
x <- matrix(c(1, 0, 0, 2, 1, 0), nrow = 3, 
    dimnames = list(1:3, LETTERS[1:2]))
x
s <- as.simple_triplet_matrix(x)
colapply_simple_triplet_matrix(s, FUN = var)
##
simplify2array(colapply_simple_triplet_matrix(s, identity))
##
crossapply_simple_triplet_matrix(s, FUN = var)
}
\keyword{algebra}
\keyword{array}
